## ----echo=F, results='hide', fig.show='hide', message=FALSE, warning=FALSE----
library(dplyr)
library(magrittr)
library(car)
library(knitr)
library(DAMisc)
opts_chunk$set(tidy='formatR', tidy.opts = list(only.comment=TRUE, comment = FALSE, width.cutoff=80),
  warning=FALSE, message=FALSE)


## ----sum , echo=TRUE-----------------------------------------------------
## adding two numbers together
2+2


## ----assign, echo=TRUE---------------------------------------------------
## show how output can be assigned to an object
X <- 10
## print the object
X


## ----assign_eq, echo=T---------------------------------------------------
## show how the = sign can also be used as the 
## assignment characterm, along with <-
X = 10
## print object
X


## ----assignsum , echo=TRUE-----------------------------------------------
## not only can numbers be assigned, but so can 
## the evaluated output of functions
X <- 4+4


## ----printX, echo=T------------------------------------------------------
## print the object X
X


## ----loadrio, echo=T-----------------------------------------------------
## load rio package
library(rio)


## ----loadformats, echo=T, eval=F-----------------------------------------
## ## install packages to deal with rio
## ## formats not currently supported in your version of R.
## install_formats()


## ----helprio, echo=T, eval=F---------------------------------------------
## ## show the formats supported by rio
## help(package="rio")


## ----readspss, echo=TRUE-------------------------------------------------
## read in data in SPSS format
spss.dat <- import("r_example.sav")


## ----printspss, echo=TRUE------------------------------------------------
## print the data frame
spss.dat


## ----spssx1, echo=T------------------------------------------------------
spss.dat$x1


x## ----mean2, echo=TRUE----------------------------------------------------
## show how arguments can be specified: 
## by order (x is the first argument to mean)
mean(spss.dat$x1)
## by name
mean(x=spss.dat$x1)
## by name, out of order
mean(na.rm=TRUE, x=spss.dat$x1)


## ----meanT, echo=TRUE----------------------------------------------------
mean(spss.dat$x1, na.rm=T)


## ----cor, echo=TRUE------------------------------------------------------
## show different types of arguments
## vectors for x and y (the first two)
## strings for use and method
cor(spss.dat$x1, spss.dat$x2, use="complete.obs",
    method="spearman")


## ----lm, echo=TRUE-------------------------------------------------------
## show formula argument to lm 
summary(lm(x1 ~ x2, data=spss.dat))


## ----readstata, echo=TRUE------------------------------------------------
## read data in Stata format
stata.dat <- import("r_example.dta")
## print data frame
stata.dat


## ----csv, echo=TRUE------------------------------------------------------
## read data in .csv format
csv.dat <- import("r_example.csv")
## print data frame
csv.dat


## ----excel, echo=TRUE----------------------------------------------------
## read data in .xlsx format, using sheet name
## to identify relevant sheet
xls.dat <- import("r_example.xlsx", which="Sheet1")
## use sheet number to identify relevant sheet
xls.dat <- import("r_example.xlsx", which=1)
## print data frame
xls.dat


## ----str, echo=TRUE------------------------------------------------------
## show the structure of the data frame. 
str(stata.dat)


## ----sum, echo=TRUE------------------------------------------------------
## show a numerical summary of the data frame
summary(stata.dat)


## ----desc, echo=TRUE-----------------------------------------------------
## load the psych package
library(psych)
## use the describe function to summarize the data
describe(stata.dat)


## ----descby, echo=TRUE---------------------------------------------------
## summarize the data in stata.dat by the grouping variable x2. 
describeBy(stata.dat, group="x2")


## ----as_fac, echo=T------------------------------------------------------
## change labelled variables into factors in
## the stata.dat data frame
stata.fdat <- factorize(stata.dat)


## ----relev, echo=TRUE----------------------------------------------------
## show the factor levels of the x2 variable
levels(stata.fdat$x2)
## change the reference level for x2 to "some"
stata.fdat$x2 <- relevel(stata.fdat$x2, ref="some")


## ----workwd, echo=TRUE---------------------------------------------------
## get the working directory in R
getwd()


## ----setwd, echo=TRUE, eval=F--------------------------------------------
## ## set the working directory in R (Mac, Linux)
## setwd("/Users/armstrod/Dropbox/IntroR")


## ----setwdwin, echo=TRUE, eval=F-----------------------------------------
## ## set the working directory in R (Windows)
## setwd("C:/Users/armstrod/Dropbox/IntroR")


## ----choosedir, echo=TRUE, eval=F----------------------------------------
## ## load the tcltk package
## library(tcltk)
## ## function to interactively choose working directory
## ## don't need this in RStudio, they already have
## ## a feature that does this.
## setwd(tk_choose.dir())


## ----choosedirwin, echo=TRUE, eval=F-------------------------------------
## ## function to interactively choose working directory
## ## don't need this in RStudio, they already have
## ## a feature that does this.
## setwd(choose.dir())


## ----writest, echo=T, eval=FALSE-----------------------------------------
## ## writing data in Stata format.
## export(stata.dat, file="stata_out.dta")


## ----x4, echo=TRUE-------------------------------------------------------
## make a vector of length 10, 
## five 0s and five 1s. 
x4 <- c(0,0,0,0,0,1,1,1,1,1)
## print the vector in x4
x4


## ----x4b , echo=TRUE-----------------------------------------------------
## read in the stat data again
stata.dat <- import("r_example.dta")
## convert labelled vectors to factors
stata.dat <- factorize(stata.dat)
## add the x4 vector as the variable 
## x4 in the stata data frame
stata.dat$x4 <- x4
## print the Stata data frame
stata.dat


## ----logs, echo=TRUE-----------------------------------------------------
## add a variable that is the log of the 
## x1 variable to hte stata data frame
## in the log_x1 variable
stata.dat$log_x1 <- log(stata.dat$x1)
## print the data frame
stata.dat


## ----rec, echo=TRUE------------------------------------------------------
## use the recode function from the car package
## note that the output prints to the console, 
## but doesn't actually change anything in the data frame.
recode(stata.dat$x1, "c(1,2)=1; c(3,4,5,6)=2")


## ----rec2 , echo=TRUE----------------------------------------------------
## save the recoded variable to the 
## variable recoded_x1 in the 
## Stata data frame
stata.dat$recoded_x1 <- recode(stata.dat$x1,
	"c(1,2)=1; c(3,4,5,6)=2")
## print the data frame
stata.dat


## ----rec3, echo=TRUE-----------------------------------------------------
## show recoding ranges of data to single values
recode(stata.dat$log_x1, "0=0; 0:1.5=1; 1.5:hi = 2")
## make a matrix from log_x1 and the recoded
## variable. 
cbind(stata.dat$log_x1, recode(stata.dat$log_x1,
	"0=0; 0:1.5=1; 1.5:hi = 2"))


## ----cut1, echo=TRUE-----------------------------------------------------
## load the RcmdrMisc package to get access to the 
## binVariable function - this is a fuller featured
## version of the cat() function. 
library(RcmdrMisc)
## load the Duncan dataset from the carData 
## package, carData is loaded automatically
## when you load the car package. 
data(Duncan)
## create a 4-group income variable that 
## has equally spaced cut points
incgroup <- binVariable(Duncan$income, bins=4, method="intervals")
## show a frequency distribution of the income 
## group variable
table(incgroup)
## create a 4-group income variable that has 
## roughly equal proportions of observations in 
## each group. 
incgroup2 <- binVariable(Duncan$income, bins=4, method="proportions")
## show a frequency distribution of the income
## group variable.
table(incgroup2)


## ----readmiss , echo=TRUE------------------------------------------------
## read in the Stata dataset with missing values
stata2.dat <- import("r_example_miss.dta")
## convert labelled variables into factors
stata2.dat <- factorize(stata2.dat)
## print the data frame
stata2.dat


## ----logical, echo=TRUE--------------------------------------------------
## show that we can evaluate for every 
## observation whether the x3 variable
## is equal to "yes" (gives TRUE or FALSE)
stata.dat$x3 == "yes"
## same operation for x2_fac equal to "none"
stata.dat$x2_fac <- factorize(stata.dat$x2)
stata.dat$x2_fac == "none"
## showing how to evaluate numerical equivalency
## note that this only works if the variable
## is numeric. 
stata.dat$x2_fac == 1
stata.dat$x1 == 2


## ----logical_which, echo=TRUE--------------------------------------------
## for each of the equalities above, 
## the observations where the equality
## evaluates to TRUE. 
which(stata.dat$x3 == "yes")
which(stata.dat$x2_fac == "none")
which(stata.dat$x2 == 1)
which(stata.dat$x1 == 2)


## ----logical2, echo=TRUE-------------------------------------------------
## subsets the stata.dat data frame 
## to keep only the observations where x1 = 1 
## and x2 = "none".  The [which(...), ] part
## of the function shows that you're keeping the 
## rows where the equality is TRUE and all of the 
## columns in the data frame.
stata.dat[which(stata.dat$x1 == 1 & stata.dat$x2 == "none"), ]


## ----isfns, echo=T-------------------------------------------------------
## shows how the "is." functions work. 
is.na(stata2.dat$x2)
is.finite(stata2.dat$x2)
is.null(stata2.dat$x2)


## ----sorted, echo=T------------------------------------------------------
## load the DescTools package so we have access
## to the Sort function.  The small-s "sort()" 
## function only sorts a vector.  The capital
## S "Sort()" function sorts a data frame. 
library(DescTools)
## Sort the Stata data based on x1 and then sort
## ties on x1 by x2.  Save the sorted data frame
## in stata.dat_sorted.
stata.dat_sorted <- Sort(stata.dat, c("x1", "x2"), decreasing=FALSE)
## print the data frame. 
stata.dat_sorted


## ----readstrikes, echo=T-------------------------------------------------
## read in the strikes data
strikes <- import("https://quantoid.net/files/rbe/strikes_small.rda")


## ----sums1, echo=T-------------------------------------------------------
## sum strike volume for Denmark using nested functions
sum(strikes[which(strikes$country == "Denmark"), "strike_vol"])


## ----sums2, echo=T-------------------------------------------------------
## load the dplyr and magrittr packages.  The dplyr package
## makes available a bunch of data management functions
## and magrittr gives access to the pipe (%>%) function. 
library(dplyr)
library(magrittr)
## show summarizing by groups using piped functions. 
strikes %>% filter(country=="Denmark") %>% select(strike_vol) %>% sum


## ----sums2a, echo=T------------------------------------------------------
## show how in the above, you could also include other
## arguments to the final function (sum in this case). 
strikes %>% filter(country=="Denmark") %>% 
    select(strike_vol) %>% sum(., na.rm=TRUE)


## ----sums3, echo=T-------------------------------------------------------
## show how the same thing as above can be accomplished 
## with the pipe and summarise functions for all countries. 
strikes %>% group_by(country) %>% 
    summarise(n_strike = sum(strike_vol))


## ----sums4, echo=T-------------------------------------------------------
## saving the country summaries to a data frame
## called tmp
tmp <- strikes %>% group_by(country) %>% 
    summarise(n_strike = sum(strike_vol))
## print the data frame
tmp


## ----ci1, echo=T---------------------------------------------------------
## load the gmodels package to get access to 
## the ci function.  
library(gmodels)
## calculate the mean and its confidence interval
## for the strike_vol variable. 
ci(strikes$strike_vol)


## ----sums5, echo=T-------------------------------------------------------
## show how summarise can be used to save multiple
## types of summaries by group - in this case, 
## the mean and the loweer and upper confidence bounds. 
tmp <- strikes %>% group_by(country) %>% 
    summarise(mean_strike = mean(strike_vol), 
    lwr = ci(strike_vol)[2], upr=ci(strike_vol)[3])
## print the object. 
tmp


## ----read, echo=T--------------------------------------------------------
## use the import() function from the rio package 
## to read in polity and ciri datasets from the web. 
polity <- import("https://quantoid.net/files/rbe/polity_small.dta")
ciri <- import("https://quantoid.net/files/rbe/ciri_small.dta")


## ----merge1, echo=T------------------------------------------------------
## show examples of each of the different
## types of merging functions.  Note 
## that all missing data match all other
## missing data on the ID variables. 
lmerge <- left_join(polity, ciri)
rmerge <- right_join(polity, ciri)
fmerge <- full_join(polity, ciri)
imerge <- inner_join(polity, ciri)
## show how the number of observations diffger.
nrow(lmerge)
nrow(rmerge)
nrow(fmerge)
nrow(imerge)


## ----dups, echo=T--------------------------------------------------------
## Identify duplicated observations based on country code and year. 
dups <- which(duplicated(lmerge[,c("ccode", "year")]))
## print duplicate observations
dups


## ----dropdups, echo=T----------------------------------------------------
## Drop duplicated observations by using the minus sign 
## to say everything but the duplicated observations
lmerge2 <- lmerge[-dups, ]



## ----readgss, echo=TRUE, warnings=FALSE, messages=FALSE------------------
## read in GSS data
library(rio)
gss <- import("GSS2012.dta")
## make factors out of the happiness
## and marital status variables
gss$happy <- factorize(gss$happy)
gss$mar1 <- factorize(gss$mar1)
## make a cross-tabulation of the 
## happiness and mar status variables
tab <- table(gss$happy, gss$mar1)
## print the table
tab


## ----addmar, echo=TRUE---------------------------------------------------
## add the marginal values to the table. 
## note, this doesn't save them in the table
## it only prints them.
addmargins(tab)


## ----proptab1, echo=TRUE-------------------------------------------------
## percentegize the table making column 
## percentages - the margin = 2 makes column
## percentages, margin = 1 makes row percentages. 
round(prop.table(tab, margin=2), 3)


## ----xt1, echo=TRUE------------------------------------------------------
## make a cross-tabulation using the 
## xtabs function.  The right-hand
## side of the formula are the variables
## that define the rows/columns.  The 
## left-hand side is the count.  If nothing
## is there, it's assumed to be 1. 
xt1 <- xtabs(~ happy + mar1, data=gss)
## print the cross-tab. 
xt1


## ----xtag, echo=TRUE-----------------------------------------------------
## import the aggregated GSS data
gss.ag <- import("GSS2012ag.dta")
## make happy and marital status into
## factors. 
gss.ag$happy <- factorize(gss.ag$happy)
gss.ag$mar1 <- factorize(gss.ag$mar1)
## look at the first 6 rows of the data. 
head(gss.ag)
## make the cross-tabulation using the 
## aggregate counts on the left-hand side
## of the formula. 
xt <- xtabs(freq ~ happy + mar1, data=gss.ag )
## print the table
xt


## ----xtab2, echo=TRUE----------------------------------------------------
## make class into a factor
gss$class <- factorize(gss$class)
## make a three-way cross-tabulation 
xt2 <- xtabs(freq ~ happy + mar1 + class, data=gss.ag)
## print the cross-tab. 
xt2


## ----ftab, echo=TRUE-----------------------------------------------------
## flatten the table from a three-dimensional 
## array into a table with two different
## variables in the rows and one in the column. 
ftable(xt2, row.vars=c("class", "mar1"))


## ----CT1, echo=TRUE------------------------------------------------------
## load the gmodels package
library(gmodels)
## use the CrossTable function to make a cross-tab. 
## note that with(data, function) uses the data
## and makes it so variables in the function don't
## need to have data$ first.  The equivalent would be
## CrossTable(gss$happy, gss$mar1). 
with(gss, CrossTable(happy, mar1))
## turn expected counts on, row proportions and 
## cell proportions off and the chi-squared 
## statistic on. 
with(gss, CrossTable(happy, mar1, expected=TRUE, prop.r=FALSE,
    prop.t=FALSE, chisq=TRUE))


## ----meas1, echo=TRUE----------------------------------------------------
## load the vcd package
library(vcd)
## summarize the output from the 
## assocstats function on the xt1 
## cross-tabulation. 
summary(assocstats(xt1))


## ----ttest1, echo=TRUE---------------------------------------------------
## recode the gss happy variable into 
## a binary variable where very happy=1 and 
## all other observed levels equal 0. 
gss$veryhappy <- recode(gss$happy, "'very happy' = 1;
    c('pretty happy', 'not at all happy') = 0; else=NA")
## Do a t-test of income on very happy using 
## the GSS data. Note that var.equal can be 
## TRUE or FALSE, indicating whether you 
## assume population variances of real income
## in the groups are the same. 
ttres <- t.test(realinc ~ veryhappy, data=gss, var.equal=F)
## print the t-test result. 
ttres


## ----lms, echo=TRUE------------------------------------------------------
## load the car package
library(car)
## Put the Duncan data in the workspace. 
## loaded with the car package
data(Duncan)
## estimate the linear regression of prestige
## on income and type from the duncan data. 
## note this just prints the coefficient. 
lm(prestige ~ income + type,data=Duncan)
## save the linear model output in the object
## called mod
mod <- lm(prestige ~ income + type,data=Duncan)
## summarize the model output. 
summary(mod)


## ----namemod, echo=TRUE--------------------------------------------------
## show the names of all of the elements
## saved in the model object. 
names(mod)


## ----contr1, echo=TRUE---------------------------------------------------
## show how the different levels of the type 
## variable from the duncan dataset will be 
## treated in a regression model. 
contrasts(Duncan$type)


## ----contr2a, echo=TRUE--------------------------------------------------
## reload the Duncan data to overwrite
## any previous changes. 
data(Duncan)
## change the reference level of the type
## variable to "prof" from "bc" and save that
## new variable in type2.
Duncan$type2 <- relevel(Duncan$type, "prof")
## estimate a new linear model with the 
## type2 variable, notice that there are
## coefficients for type2bc and type2wc.
lm(prestige ~ income + type2, data=Duncan)


## ----contr3, echo=TRUE---------------------------------------------------
## make a copy of the type variable 
## in type3 in the Duncan data. 
Duncan$type3 <- Duncan$type
## change the contrasts to the 
## sum-to-zero type. 
contrasts(Duncan$type3) <- 'contr.sum'
## show model output. 
lm(prestige ~ type3, data=Duncan)


## ----diagTests, echo=TRUE------------------------------------------------
## estimate a linear model of prestige on 
## income, education and type from the Duncan data. 
mod <- lm(prestige ~ income + education + type, data=Duncan)
## Test for heteroskedasticity. 
ncvTest(mod, var.formula = ~ income + education + type)
## Test for outliers using studentized residuals.
outlierTest(mod)


## ----av1, echo=TRUE, fig.show='hide',  fig.height=8, fig.width=8---------
## make the added variable plot (aka partial regression plot)
## for all variables at once.  Note you can also 
## use the avPlot() function (without the s) to make
## the plot for a single term. 
avPlots(mod)


## ----cr1, echo=TRUE, fig.show='hide', fig.height=8, fig.width=8----------
## make the component + residual (aka partial residual) 
## plot.  Note that you can use the crPlot() 
## function to make the plot for a single variable. 
crPlots(mod)


## ----rp1, echo=TRUE, fig.show='hide', fig.height=8, fig.width=8----------
## make a series of residual plots. 
## this gives y vs x and y-hat vs x
## plots along with curvature stats.
residualPlots(mod)


## ----ip1, echo=TRUE, fig.show='hide', fig.height=8, fig.width=8----------
## make an influence bubble plot for the model
## points are at the nexus of studentized residuals
## and hat values and sized proportionately to 
## Cook's D. 
influencePlot(mod)


## ----pred1, echo=TRUE----------------------------------------------------
## reload the Duncan data to remove any
## previous changes.  
data(Duncan)
## change the reference category of the 
## type variable to 'prof'. 
Duncan$type <- relevel(Duncan$type, "prof")
## estimat ethe linear model of prestige
## on income and type from the Duncan data
mod <- lm(prestige ~ income + type, data=Duncan)
## summarize the model object to get 
## SEs, t-stats and p-values. 
summary(mod)
## show two different ways to get fitted values
## for all observations 
## first, access the 'fitted' element of the
## model object. 
mod$fitted
## second use the extractor function fitted()
## to return the fitted values. 
fitted(mod)


## ----pred2, echo=TRUE----------------------------------------------------
## predict will also return the fitted 
## values for the model, but can do more. 
pred <- predict(mod)
## show the first 6 predictions.
head(pred)
## save predictions along with standard
## errors
pred.se <- predict(mod, se.fit=TRUE)
## the predicted values themselves are 
## in the 'fit' element of the predictions. 
head(pred.se$fit)
## the standard errors are in the 'se.fit'
## element of the predictions
head(pred.se$se.fit)
## show how to produce confidence intervals
## for predictions
pred.mean.ci <- predict(mod, interval="confidence")
## print the first six rows of the predictions
## with confidence intervals. Note, these 
## are confidence intervals for the regression
## line (or confidence intervals for a mean 
## prediction). 
head(pred.mean.ci)
## calculate prediction intervals, these are 
## confidence intervals for a hypothetical future
## single observation and thus will be bigger. 
pred.ind.ci <- predict(mod, interval="prediction")
## show the first six rows of the data with prediction 
## intervals. 
head(pred.ind.ci)


## ----pred3, echo=TRUE----------------------------------------------------
## make a new data frame with a single 
## observsation where income = 50 and 
## type is "bc". 
newdat <- data.frame(
	income = 50,
	type = "bc")
## generate predictions and confidence intervals
## for this single hypothetical observation. 
predict(mod, newdat, interval="confidence")


## ----pred4, echo=TRUE----------------------------------------------------
## generate three hypothetical observations
## where income = (40, 50 and 60) and 
## type is held constant at 'bc'.  
## Note, you could also do this with the 
## expand.grid() function with: 
## expand,.grid(income=c(40,50,60), type="bc")
newdat <- data.frame(
	income = c(40, 50, 60),
	type = c("bc", "bc", "bc"))
## show predictions and confidence intervals for
## these three hypothetical observations. 
predict(mod, newdat, interval="confidence")


## ----lh1, echo=TRUE------------------------------------------------------
## test the hypothesis that the income coefficient
## from the model 'mod' is equal to one. 
linearHypothesis(mod, "income = 1")


## ----lh2, echo=TRUE------------------------------------------------------
## test the hypothesis that the coefficients
## for the bc dummy and wc dummyf rom the model
## 'mod' are the same. 
linearHypothesis(mod, "typebc=typewc")


## ----lh3, echo=TRUE------------------------------------------------------
## tes tthe hypothesis that both the
## coefficients for the bc dummy and 
## the wc dummy are simultaneously 
## (or jointly) zero. 
linearHypothesis(mod, c("typebc=0", "typewc=0"))


## ----anova, echo=TRUE----------------------------------------------------
## use the Anova() function from the car package
## to calculate Type II Anova tests for the 
## terms in mod. 
Anova(mod)


## ----catvar, echo=TRUE---------------------------------------------------
## load the car package again, technically
## since it's already loaded you wouldn't need
## to do this again.  It's more to identify that
## the Ornstein data come from the car package,
library(car)
## load the Ornstein data. 
data(Ornstein)
## estimate a linear model of interlocks on 
## nation, sector and the log of assets,. 
## Notice that we can embed the functional 
## transformation within the model. 
mod <- lm(interlocks ~ nation + sector +
    log(assets), data=Ornstein)
## summarize the regression model. 
summary(mod)


## ----glht1, echo=TRUE----------------------------------------------------
## load the multcomp package
library(multcomp)
## use the glht() function to generate
## all of the pairwise combinations of 
## coefficients for the sector variable. 
glht(mod, linfct=mcp(sector = "Tukey"))


## ----fp1, echo=TRUE, fig.show='hide', fig.height=8, fig.width=8----------
## load the factorplot package
library(factorplot)
## use factorplot to calculate all of the pairwise
## combinations of the sector variable. 
fp <- factorplot(mod, factor.var = "sector")
## plot the factorplot object. 
plot(fp)

## ----inter, echo=TRUE----------------------------------------------------
## estimate a linear model of prestige on 
## the interaction of income and education
## along with type.  Note that using the 
## asterisk will include both the "main effects"
## of income and education along with the 
## product regressor
mod <- lm(prestige ~ income*education + type, data= Duncan)


## ----instdamisc, echo=T, eval=FALSE--------------------------------------
## ## load the devtools package
## library(devtools)
## ## install the damisc package from my
## ## github repository.
## install_github("davidaarmstrong/damisc")


## ----intfun2, echo=TRUE, fig.show='hide', fig.height=4, fig.width=8------
## load the DAMisc package
library(DAMisc)
## create a conditional effects plot for the income
## and education interaction. Note that by default
## this plots the display to the console.  You can have 
## it save the plots to other types of files, too. 
DAintfun2(mod, c("income", "education"), rug=F, hist=T,
    scale.hist=.3)


## ----int2, echo=TRUE-----------------------------------------------------
## estimate the regression of prestige on the 
## income and type interaction along with education. 
## This time, we're using the Prestige data. 
mod2 <- lm(prestige ~ income*type + education, data=Prestige)


## ----slopes1, echo=TRUE, fig.show='hide'---------------------------------
## Show the simple slopes of income conditional 
## on occupation type 
intQualQuant(mod2,
    c("income", "type"),
    type="slopes",
    plot=FALSE)


## ----slopes2, echo=TRUE, fig.show='hide', fig.height=8, fig.width=8------
## Plot the simple slopes of income conditional 
## on occupation type 
intQualQuant(mod2,
    c("income", "type"),
    type="slopes",
    plot=TRUE)


## ----slopes1a, echo=TRUE, fig.show='hide'--------------------------------
## show the effect of type conditional on 
## income in numerical form. 
intQualQuant(mod2,
    c("income", "type"),
    n = 3,
    type="facs",
    plot=FALSE)


## ----slopes2a1, echo=TRUE, fig.show='hide'-------------------------------
## plot the conditional effect of type based on income. 
## note that this makes a plot that is stored in the 
## object 'p' and it makes one panel for every pair of 
## levels of the factor variable. 
p <- intQualQuant(mod2, c("income", "type"), type="facs", plot=TRUE)

## ----slopes2a, echo=TRUE, fig.show='hide', fig.height=4, fig.width=9-----
## update the plot so that there are three columns and
## one row of displays. 
update(p, layout=c(3,1), aspect=1)


## ----modtrans, echo=TRUE-------------------------------------------------
## Estimate the linear regression model of 
## prestige on log-transformed income, education
## and type 
transmod <- lm(prestige ~ log(income) + education + type, data=Prestige)


## ----transeff, echo=TRUE, fig.show='hide', fig.height=6, fig.width=6-----
## load the effects package
library(effects)
## use predictorEffect to figure out the effect of 
## income in the transmod model.  
## xlevels tells the function to pick 25 values
## of income at which to make predictions.
## The result is then plotted with the plot() function
plot(predictorEffect("income", transmod, xlevels=25), main="")


## ----transeffvl, echo=TRUE, fig.show='hide', fig.height=6, fig.width=6----
## load the visreg package
library(visreg)
## show the effect of income in the transmod object. 
## by default, lattice is used. 
visreg(transmod, "income")


## ----transeffvg, echo=TRUE, fig.show='hide', fig.height=6, fig.width=6----
## make the same plot as above, but using 
## a ggplot, with gg=TRUE.
visreg(transmod, "income", gg=TRUE)


## ----modtrans2, echo=TRUE------------------------------------------------
## make another model where income is 
## transformed.  Here, the point is to show
## that any transformation can happen within the 
## I() construct. 
transmod2 <- lm(prestige ~ I(1/income^2) + education + type, data=Prestige)


## ----transeff2, echo=TRUE, fig.show='hide', fig.height=6, fig.width=6----
## Again, we use predictorEffect() to calculate
## the effect of income in this new model and 
## plot its result using the plot() function. 
plot(predictorEffect("income", transmod2, xlevels=25), main="")


## ----polymod, echo=TRUE--------------------------------------------------
## Estimate a model like the ones above that has 
## a third-degree polynomial in income.  
polymod <- lm(prestige ~ poly(income, 3) + education + type, data=Prestige)
## summarize the model
summary(polymod)


## ----polyeff, echo=TRUE, fig.show='hide', fig.height=6, fig.width=6------
## use predictorEffect() to calculate the effect of income from the 
## polynomial model.  Plot it using the plot() function. 
plot(predictorEffect("income", polymod, xlevels=25), main="")


## ----anovas, echo=TRUE---------------------------------------------------
## use the lower-case a anova() function to get the 
## type I sequential sums of squares or to test
## two models against each other. 
anova(mod2)
## use capital A Anova() to conduct tests with 
## type II and III sums of squares.  
## Here's a Type II test
Anova(mod2)
## Here's a Type III test
Anova(mod2, type="III")


## ----nest1, echo=TRUE----------------------------------------------------
## Make a data set that only has observations
## with complete information on income, 
## prestige, education, women and type.  
## The complete.cases() function finds all  observations
## that have no missing data in the function.  Wrapping
## the output of that function in Prestige[ ,] means
## that R will only keep the observations of the Prestige
## data where observations re complete on the input data.
tmp <- Prestige[complete.cases(Prestige[,c("income", "prestige",
    "education", "women", "type")]), ]
## estimate a set of nested models - m1 is nested in m2.  
m1 <- lm(prestige ~ income + education, data=tmp)
m2 <- lm(prestige ~ income + education + women + type, data=tmp)
## use anova() to test if the addition of women and type
## make the model significantly better.  One important
## requirement is that the models be on exactly the 
## same observations. 
anova(m1, m2, test="F")


## ----nnest1, echo=TRUE---------------------------------------------------
## estimate two non-nested models. 
m1 <- lm(prestige ~ education + type, data=tmp)
m2 <- lm(prestige ~ income + women, data=tmp)
## load the lmtest package
library(lmtest)
## estimate the encompassing test
## of nonnested models
encomptest(m1, m2, Prestige)
## estimate the J test of 
## nonnested models 
jtest(m1, m2, Prestige)
## estimate the cox test
## of nonnested models
coxtest(m1, m2, Prestige)


## ----nnest2, echo=TRUE---------------------------------------------------
## load the games package
library(games)
## estimate the vuong test 
## of nonnested models
vuong(m1, m2)
## estimate Clarke's distribution
## free test of nonnested models
clarke(m1, m2)


## ----chile, echo=TRUE----------------------------------------------------
## put the Mroz data in your workspace
data(Mroz)
## estimate a logistic regression model of labor-force
## participation.  Note that we use the asinh 
## (inverse hyperbolic sine) transformation, which works
## like the log, but permits nonpositive values. 
## The logit model uses link="logit"
logit.mod <- glm(lfp ~ k5 + k618 + log(age) + wc + lwg*asinh(inc), 
    data=Mroz, family=binomial(link="logit"))
## estimate the same model as above, but using 
## link="probit" for the probit model
probit.mod <- glm(lfp ~ k5 + k618 + log(age) + wc + lwg*asinh(inc), 
    data=Mroz, family=binomial(link="probit"))


## ----eff1, echo=TRUE-----------------------------------------------------
## load the effects package (though we had 
## previously done this so it isn't strictly
## necessary at this point)
library(effects)
## Calculate the effect of the age variable
## in the logit model.
eff <- predictorEffect("age", logit.mod, focal.levels=10)
## summarize the effect - will show fitted values along
## with confidence intervals. 
summary(eff)


## ----ncoef1, echo=T------------------------------------------------------
## get the names of the logistic model coefficients. 
## this is required to be able to set of these
## regressors in calculating the effects. 
names(coef(logit.mod))


## ----eff1a, echo=T-------------------------------------------------------
## calculate the effect of age as above, but 
## set the values at which other variables
## will be held constant. 
e <- predictorEffect("age", logit.mod, focal.levels=5, 
      fixed.predictors=list(
          given.values=c(k5 = 0, 
          k618 = 2, "asinh(inc)" = asinh(10))))
## the resulting object "e" is a list whose elements
## are, in part, "fit", "lower" and "upper".  We may
## want to turn these into a matrixx for easier viewing.
## to make a matrix with predictions and confidence
## intervals, we use the do.call() function which takes
## as its first argument a function (in this case the column
## binding function) and as its second arguments a list of
## arguments to be passed to the first argument, in this case
## it is a list of vectors to be passed to the cbind() function. 
do.call(cbind, summary(e)[c("effect", "lower", "upper")]) 
## note the result here is the same as if we had done: 
## cbind(summary(e)$effect, summary(e)$lower, summary(e)$upper))


## ----glmchange, echo=TRUE------------------------------------------------
## calculate the first differences for all variables in logit.mod 
## for a standard deviation change in each variable holding the 
## others constant (using the MER approach).  
glmChange(logit.mod, Mroz, diffchange="sd", sim=TRUE)
## calculate the first differences for a standard deviation
## change in age age in the logit.mod object.  This uses the 
## AME approach - keeping each observations values on the 
## other variables as they are. 
glmChange2(logit.mod, "age", Mroz, change="sd")


## ----aep, echo=TRUE, fig.show='hide', fig.height=8, fig.width=8----------
## use the AME approach to make the effects plot for age. 
aveEffPlot(logit.mod, "age", Mroz)


## ----aep2, echo=TRUE, fig.show='hide', fig.height=8, fig.width=8---------
## use the aveEffPlot() function to caluclate the 
## effect of age using the AME appraoch and save 
## the resulting predicted probabilities and confidence
## intervals to the object aep2. 
aep2 <- aveEffPlot(logit.mod, "age", Mroz, plot=F)
## load the ggplot2 package
library(ggplot2)
## use ggplot to make the effects graph.  
## the variables that will be used in the plot 
## go in the aes() argument to the ggplot() function. 
## geom_ribbon() makes the confidence envelope, 
##    alpha is the transparency parameter
## geom_line() plots the predicted probabilities themselves
## xlab() and ylab() alter the x- and y-axis labels. 
ggplot(aep2, aes(x=s, y=mean, ymin = lower, ymax=upper)) + 
    geom_ribbon(fill="gray75", alpha=.5) + 
    geom_line() + 
    xlab("Age") + 
    ylab("Predicted Probability of\nParticipating in the Labor Force") 


## ----binfit, echo=TRUE---------------------------------------------------
## print fit measures for the logistic
## regression model
binfit(logit.mod)
## print the proportional reduction in 
## error and the ePRE for the logistic
## regression model
pre(logit.mod)


## ----inteffl, echo=TRUE, fig.show='hide', fig.height=6, fig.width=6------
## plot the interction effect using
## the effects package
plot(effect("lwg*asinh(inc)", logit.mod, xlevels=6), type="response", main="")


## ----secdiff, echo=T-----------------------------------------------------
## create the second difference for each 
## individual observation and the 
## difference of the average second 
## difference using the secondDiff()
## function from the DAMisc package
secd <- secondDiff(logit.mod, c("lwg", "inc"), Mroz) 
## calculate the proportion of times
## that the average second difference
## is bigger than 0.  You could take
## the smaller of this number and one 
## minus this number as a one-sided p-value
## on whether the second difference is 
## different from zero. 
mean(secd$ave > 0)
## calculates the proportion of observations
## for which the p-value for the individual
## second difference is less than 0.05
mean(secd$ind$pval < .05)


## ----polr, echo=TRUE-----------------------------------------------------
## load the ordinal and DAMisc packages. 
## technically we could do without loading
## the DAMisc package because it was 
## already loaded above. 
library(ordinal)
library(DAMisc)
## bring the france data into the 
## current workspace
data(france)
## estimate the ordered logistic regression
## model with clm from the ordinal package. 
ologit.mod <- clm(retnat ~ lrself + male + age, data=france)
## summarize the ordered logit model.
summary(ologit.mod)


## ----eff2, echo=TRUE-----------------------------------------------------
## calculate the effect of the left-right
## self-placement variable with the 
## effect() function from the effects
## package
eff2 <- effect("lrself", ologit.mod, xlevels=list(lrself=c(1,5,10)))
## summarize the effect object to get 
## predicted probabilities and 
## confidence intervals
summary(eff2)


## ----oej, echo=T, fig.show='hide', fig.height=8, fig.width=8-------------
## plot the effect in juxtaposed panels
plot(effect("lrself", ologit.mod, xlevels=25))

## ----oes, echo=T, fig.show='hide', fig.height=8, fig.width=8-------------
## plot the effect with superposed lines
plot(effect("lrself", ologit.mod, xlevels=25), multiline=TRUE)

## ----oek, echo=T, fig.show='hide', fig.height=8, fig.width=8-------------
## plot the effect with a stacked area chart
plot(effect("lrself", ologit.mod, xlevels=25), style="stacked")


## ----gologit, echo=T-----------------------------------------------------
## estimate the generalized ordered logit
## model where the variable lrself is 
## allowed to have different coefficients 
## across the categories. 
gologit.mod <- clm(retnat ~   male + age, nominal = ~ lrself, 
    data=france)
## summarize the generalized ordered logit
## model. 
summary(gologit.mod)


## ----gologit_test, echo=T------------------------------------------------
## Test the generalized ordered logit
## model against the ordered logit model
## with anova(), which does a LR-test. 
anova(ologit.mod, gologit.mod)


## ----ordift, echo=T------------------------------------------------------
## load the MASS package
library(MASS)
## estimate the ordered logit model
## with the polr function. 
polr.mod <- polr(retnat ~ lrself + male + age, data=france)
## get scalar measures of fit for 
## the ordered logit model. 
ordfit(polr.mod)


## ----orddiff, echo=T-----------------------------------------------------
## calculate the change in predicted 
## probability for a SD change in each 
## independent variable using MER method
print(ordChange(polr.mod, france, diffchange="sd", sim=TRUE))
## calculate the change in predicted 
## probability for a SD change in left-right 
## self placement using AME method
print(ordChange2(polr.mod, "lrself", france, diffchange="sd"))


## ----mnl, echo=TRUE------------------------------------------------------
## load the nnet package
library(nnet)
## bring the Chile data into your workspace.
data(Chile)
## estimste the MNL model using the 
## multinom function. 
mnl.mod <- multinom(vote ~ log(income) + age + sex + education, 
    data=Chile)
## use the mnlSig() function from DAMisc 
## to summarize the coefficients from 
## the MNL model
mnlSig(mnl.mod)


## ----eff3, echo=TRUE-----------------------------------------------------
## find the effect of education on the 
## probability of being in each category
eff3 <- predictorEffect("education", mnl.mod, focal.levels=5)
## summarize the effect to get 
## predicted probabilities and 
## confidence intervals.
summary(eff3)


## ----mlogit1, echo=T-----------------------------------------------------
## reload the Chile data
data(Chile)
## load the mlogit package
library(mlogit)
## create an mlogit.data object
## that puts the data in the 
## appropriate format to use the mlogit
## function. 
chile.ml <- mlogit.data(Chile, shape="wide", choice="vote")
## estimate the model using the mlogit() 
## function. 
mnl.mod2 <- mlogit(vote ~ 0 | log(income) + age + sex + education, 
    data= chile.ml)
## summarize the mlogit model. 
summary(mnl.mod2)


## ----mlogit_lh, echo=T---------------------------------------------------
## test hypotheses that the effect of 
## age on the choice between Yes and Abstain
## (the reference) is the same as the effect
## of age on the choice between No and Abstain. 
linearHypothesis(mnl.mod2, "N:age = Y:age")


## ----mixedlog, echo=T----------------------------------------------------
## load the car data from the mlogit package. 
data("Car", package = "mlogit")
## create the mlogit.data object from the Car
## data that identifies the appropriate 
## variables that vary by choice. 
Car <- mlogit.data(Car, varying = 5:70, shape = "wide", sep = "",
                   choice = "choice", alt.levels = 1:6)


## ----carmod, echo=T------------------------------------------------------
## estimate the mlogit model with choice 
## varying variables (cost and price).  This
## is the conditional logit model. 
carmod <- mlogit(choice ~ cost+price | college + coml5, data=Car)
## summarize the model. 
summary(carmod)


## ----carme, echo=T-------------------------------------------------------
## find the effect of the "college" variable
## using the effects() [note the "s" at the end] 
## function from the mlogit package.
carme <- effects(carmod, covariate="college", type="rr", data=Car)
## get the effects and standard deviation. 
me <- round(cbind(colMeans(carme), apply(carme, 2, sd)), 3)
## change the column names of the result matrix
colnames(me) <- c("Effect", "SD")
## print the result matrix. 
me


## ----read_surv, echo=T---------------------------------------------------
## import the cabinet data into the
## object called dat
dat <- import("cabinet2.dta")
## turn investitute into a factor
dat$invest <- factorize(dat$invest)
## create a Survey object with the 
## appropriate treatment of censoring. 
S <- Surv(dat$durat, dat$censor)


## ----surv1, echo=T, fig.height=8, fig.width=8, fig.show='hide'-----------
## load the survival package
library(survival)
## calculate the survival curve for the 
## outcome variable. 
out <- survfit(S ~ 1)
## plot the survival curve. 
plot(out)    


## ----surv2, echo=T, fig.height=8, fig.width=8, fig.show='hide'-----------
## find the survival functions for 
## cabinets conditional on their 
## investiture status. 
out2 <- survfit(
    Surv(durat, censor) ~ invest, 
    data=dat)
## plot the survival curves for 
## cabinets in investiture and not separately. 
plot(out2, col=c("black", "red"))
## add a legend to the plot. 
legend("topright", c("No Investiture", 
    "Investiture"), col=c("black", "red"), 
    lty = c(1,1), inset=.01)    


## ----smod1, echo=T-------------------------------------------------------
## do a parametric survival model 
## on cabinet duration as a function of
## investitute using the exponential 
## distribution.
mod <- survreg(Surv(durat, censor) ~ invest, data=dat, 
    dist="exponential")
## summarize the model. 
summary(mod)


## ----parsurv1, echo=T, fig.height=8, fig.width=8, fig.show='hide'--------
## plot the effect of investiture on survival. 
termplot(mod, dat, term=1, se=2)


## ----survmod2, echo=T----------------------------------------------------
## add polarization to our exponential 
## survival model. 
mod2 <- survreg(Surv(durat, censor) ~ invest + polar, data=dat, 
    dist="exponential")


## ----parsurv2, echo=T, fig.height=8, fig.width=8, fig.show='hide'--------
## create a hypothetical dataset where 
## invest = 0 and polariztion = (5, 15, 25). 
tmp <- data.frame(
    invest = factor(c(0), levels=c(0,1), 
    labels=levels(dat$invest)), 
    polar = c(5,15,25))
## generate a sequence of probabilities
## from 0,99 to 0 by -.01s, 
p <- seq(.99,0,by=-.01)
## Predict survival probabilities 
## based on the model above. 
preds <- predict(mod2, newdata=tmp, 
    type="quantile", p=p)
## collect the results 
plot.data <- data.frame(
    p.fail = rep(p, each=nrow(preds)), 
    time = c(preds))
## add variables to the prediced values 
## dataset based on conditions used to 
## generate the results. 
plot.data$polar <- rep(
    tmp$polar, ncol(preds))
## take out any predicted duration of 
## at least 60 months, so the few 
## outliers don't reduce resolution 
## in the plot. 
plot.data <- plot.data[
    which(plot.data$time < 60), ]
## make a plot of the survival probabilities. 
ggplot(plot.data, aes(x=time, y=1-p.fail, 
        colour=as.factor(polar))) +
    geom_line() + 
    scale_colour_discrete(name="Polarization")


## ----weib1, echo=T-------------------------------------------------------
## estimate the same model as above, but 
## assuming Weibull distributed errors. 
mod3 <- survreg(Surv(durat, censor) ~ invest + polar, data=dat, 
    dist="weibull")
## summarize the model. 
summary(mod3)


## ----cph1, echo=T--------------------------------------------------------
## us a cox proportional hazard model 
## to estimate the relationships
mod <- coxph(Surv(durat, censor) ~ invest + polar, data=dat)
## sumarize the coxph model
summary(mod)


## ----cphfig, echo=T, fig.show='hide', fig.height=8, fig.width=8----------
## load the survminer package
library(survminer)
## re-create the same 
## hypothetical data frame
## we made above. 
tmp <- data.frame(
    invest = factor(c(0), levels=c(0,1), 
    labels=levels(dat$invest)), 
    polar = c(5,15,25))
## generate the survival probabilities
## from the coxph model for the 
## hypothetical data
sfmod <- survfit(mod, newdata=tmp)
## plot the survival curves using the 
## ggsurvplot function from the survminer
## package
ggsurvplot(sfmod, data=tmp)$plot + 
    scale_colour_discrete(name="Polarization", labels = c("5", "15", "25")) + 
    scale_fill_discrete(name="Polarization", labels = c("5", "15", "25"))


## ----mldat, echo=T-------------------------------------------------------
## read in the mlbook data
mlbook <- import("https://quantoid.net/files/rbe/mlbook2_r.dat", header=T)


## ----mlmod1, echo=T------------------------------------------------------
## load thhe lme4 package
library(lme4)
## estimate the multilevel linear
## model of langpost on individual 
## iq verbal and school iq verbal scores. 
## use the school number as the level-2 
## identifier. 
m1 <- lmer(langpost ~ iq_verb + sch_iqv + (1|schoolnr), data=mlbook)
## summarize the multilevel model
summary(m1)


## ----mlmod1t, echo=T-----------------------------------------------------
## load the lmerTest package
library(lmerTest)
## re-estiamte the model again, this time
## lmer comes from the lmerTEst package, which
## will allow the summary to produce p-values
m1 <- lmer(langpost ~ iq_verb + sex + ses + (1|schoolnr), data=mlbook)
## summarize the model to get the 
## variable degrees of freedom and p-values
summary(m1)


## ----mlmod1eff, echo=T, fig.show='hide', fig.height=8, fig.width=8-------
## calculate the effect of individual 
## verbal iq from the model
e <- predictorEffect("iq_verb", m1)
## plot the effect. 
plot(e, main = "", xlab="Verbal IQ Score", 
    ylab = "Predicted Language Test Score")


## ----mlmod2, echo=T------------------------------------------------------
## add in several other variables to the 
## multilevel model.  Include the cross-level
## interaction of individual verbal iq and 
## school-level verbal iq. 
m2 <- lmer(langpost ~ iq_verb*sch_iqv + sex + ses + sch_ses +  minority + 
             sch_min +  (1+iq_verb|schoolnr), data=mlbook)
## summarize the model
summary(m2)


## ----mlmod2eff, echo=T, fig.show='hide', fig.height=8, fig.width=8-------
## calculate the conditional effect of 
## individual level verbal iq conditional 
## on school-level verbal iq. 
e <- effect("iq_verb*sch_iqv", m2, 
    xlevels=list(sch_iqv = c(-3,-2,-1,0,1,2), 
    iq_verb = seq(-8, 6, length=25)))
## plot the effect. 
plot(e,  as.table=T, 
    main = "", xlab="Verbal IQ Score", 
    ylab = "Predicted Language Test Score")


## ----addGmean, echo=T----------------------------------------------------
## create a function that adds group-means to the data. 
## define the function and its inputs.  Note, that you could
## also calculate the group median or any other summary function. 
addGmean <- function(vars, groupvars, data, stat = "mean", na.rm=TRUE, ...){
    # makes sure the plyr package is loaded
    require(plyr)
    # creates a formula from the vector of input variables. 
    form <- as.formula(paste(" ~ ", paste(vars, collapse="+"), sep=""))
    # creates the design matrix from the formula. 
    X <- model.matrix(form, data)[,-1]
    # creates a unique grouping variable based on 
    # the input grouping variables. 
    g <- apply(data[, groupvars, drop=FALSE], 1, paste, collapse=":")
    # calculates the statistic for each variable for each group
    bys <- by(X, list(g), apply, 2, stat, na.rm=na.rm, ...)
    # creates a matrix from the results. 
    bys <- do.call(rbind, bys)
    # adds a variable "g" to the data set that is just a copy of 
    # the grouping variable
    data$g <- g
    # converts the matrix of results into a data frame. 
    meandata <- as.data.frame(bys)
    # appends "_gm" to the end of each variable name 
    names(meandata) <- paste(names(meandata), "_gm", sep="")
    # puts the grouping variable into the dataset. 
    meandata$g <- rownames(meandata)
    # merges the individual and group-level data together
    out <- join(data, meandata, by="g")
    # returns the merged data, 
    return(out)
}


## ----mlgm, echo=T--------------------------------------------------------
## use the function defined above to create the group-
## means of the variables identified below. 
new <- addGmean(c("iq_verb", "ses", "sex", "minority"), 
    "schoolnr", mlbook)
## create within transformed variables by 
## subtracting the group-mean from the individual
## level variables. 
new$iqv_w <- new$iq_verb - new$iq_verb_gm
new$sex_w <- new$sex - new$sex_gm
new$ses_w <- new$ses - new$ses_gm
new$min_w <- new$minority - new$minority_gm
## estimate the multilevel model with both the within and between
## variables in the model. 
m3 <- lmer(langpost ~ iqv_w + iq_verb_gm + sex_w + sex_gm +
     min_w + minority_gm + ses_w + ses_gm +  (1|schoolnr), data=new)
## summarize the model
summary(m3)


## ----mllh, echo=T--------------------------------------------------------
## for iq verbal and minority, test to see 
## whether the within and between effects
## are the same. 
linearHypothesis(m3, "iqv_w = iq_verb_gm")
linearHypothesis(m3, "min_w = minority_gm")


## ----mlgm2, echo=T-------------------------------------------------------
## Re-estimate the model with random coefficients for
## verbal IQ and a cross-level interaction between individual 
## and group-level verbal iq. 
m4 <- lmer(langpost ~ iqv_w*iq_verb_gm + sex_w + sex_gm +
     min_w + minority_gm + ses_w + ses_gm +  (1 + iqv_w|schoolnr), 
     data=new)
## summarize the model
summary(m4)



## ----tscs1, echo=T-------------------------------------------------------
## read in some Time-series Cross-sectional data
tscs <- import("ajpsdata.dta")
## estimate a model with partially-crossed 
## random effects for country and year. 
## This is the random-effects version of a 
## two-way effects mdoel. 
mod <- lmer(sd ~ cwarcow + iwarcow + milcontr + logpop + 
    logpcgnp + demtri + (1|ccode) + (1|year), data=tscs)
summary(mod)


## ----br1, echo=T, eval=F-------------------------------------------------
## ## load the brms package
## library(brms)
## ## estimate a bayesian multilevel model
## ## that uses the same syntax as lmer.
## m3a <- brm(langpost ~ iqv_w + iq_verb_gm + sex_w + sex_gm +
##      min_w + minority_gm + ses_w + ses_gm +  (1|schoolnr), data=new, chains=2)

## ----br1e, echo=F--------------------------------------------------------
## load the brms package
library(brms)
## Since I didn't want to always run the brm function 
## because it takes a long time, I loaded the 
## saved workspace that has the brm results. 
load("brms1.rda")


## ----br2, echo=T---------------------------------------------------------
## summarize the brm model
summary(m3a)
## do a Bayesian test of the equality of 
## between and within effects. 
hypothesis(m3a, "iqv_w = iq_verb_gm")


## ----measdat, echo=T-----------------------------------------------------
## load the lavaan package
library(lavaan)
## put the PoliticalDemocracy data into the workspace
data(PoliticalDemocracy)
## load the psych package
library(psych)
## do an exploratory factor analysis of the 
## political democracy data for 1965. 
fa_dem65 <- fa(PoliticalDemocracy[,c("y5", "y6", "y7", "y8")], 
    SMC=TRUE, nfactors=1, fm="wls", scores="regression", cor="mixed")
## print the factor analysis result. 
fa_dem65


## ----fas, echo=T---------------------------------------------------------
## estimate the exploratory factor analysis for each of the 
## three Bollen variables - ind60, dem60, dem65.
fa_dem60 <- fa(PoliticalDemocracy[,c("y1", "y2", "y3", "y4")], 
    SMC=TRUE, nfactors=1, fm="wls", scores="regression", cor="mixed")
fa_ind60 <- fa(PoliticalDemocracy[,c("x1", "x2", "x3")], 
    SMC=TRUE, nfactors=1, fm="wls", scores="regression", cor="mixed")
## put the estimated factor scores back into the 
## Political Democracy data frame. 
PoliticalDemocracy$dem65 <- c(fa_dem65$scores)
PoliticalDemocracy$dem60 <- c(fa_dem60$scores)
PoliticalDemocracy$ind60 <- c(fa_ind60$scores)


## ----lav1, echo=T--------------------------------------------------------
## load the lavaan package
library(lavaan)
## make the lavaan confirmatory factor model. 
## Note that the =~ identifies latent variables 
## on the left-hand side and indicators on the 
## right-hand side.  The ~ indicates regressions 
## that could include observed and latent variables. 
## the ~~ indicate variances or covariances that 
## are allowed to be estimated by the model. 
model <- '
  # measurement model
    ind60 =~ x1 + x2 + x3
    dem60 =~ y1 + y2 + y3 + y4
    dem65 =~ y5 + y6 + y7 + y8
  # regressions
    dem60 ~ a1*ind60
    dem65 ~ b1*ind60 + b2*dem60
  # residual correlations
    y1 ~~ y5
    y2 ~~ y4 + y6
    y3 ~~ y7
    y4 ~~ y8
    y6 ~~ y8
'
## fit the model with the sem() Function. 
fit <- sem(model, data=PoliticalDemocracy, mimic="MPlus")
## summarize the fitted SEM
summary(fit)


## ----lavp, echo=T, eval=F------------------------------------------------
## ## load the lavaanPlot package
## library(lavaanPlot)
## ## Make a path diagram based on the model.
## lavaanPlot(model=fit)


## ----lav2, echo=T--------------------------------------------------------
## load the lavaan package again, though 
## we technically didn't have to do this since
## we already loaded it above. 
library(lavaan)
## Generate the same model, but the indirect
## and total effects of ind60 are included
## at the bottom.  
med.model <- '
  # measurement model
    ind60 =~ x1 + x2 + x3
    dem60 =~ y1 + y2 + y3 + y4
    dem65 =~ y5 + y6 + y7 + y8
  # regressions
    dem60 ~ a1*ind60
    dem65 ~ b1*ind60 + b2*dem60
  # residual correlations
    y1 ~~ y5
    y2 ~~ y4 + y6
    y3 ~~ y7
    y4 ~~ y8
    y6 ~~ y8
  # Indirect Effect of ind60
   ab := a1*b2
  # Total Effect of ind60
   total := (a1*b2) + b1
'
## fit the model that includes definitions of 
## the indirect and total effect. 
fit2 <- sem(med.model, data=PoliticalDemocracy, mimic="MPlus")
## summarize the model object. 
summary(fit2)


## ----lavfit, echo=T------------------------------------------------------
## calculate fit measures for the SEM.
fitMeasures(fit)


## ----rob1a, echo=T-------------------------------------------------------
## load the stargazer, swidwich and lmtest 
## packages
library(stargazer)
library(sandwich)
library(lmtest)
## estimate a linear model of the log
## of strike volume on inflation, 
## unemployment and repression. 
mod1 <- lm(log(strike_vol + 1) ~ inflation +
    unemployment + sdlab_rep, data=strikes)
## get the coefficient table using HC3 standard
## errors to make t-stats and p-values.
coeftest(mod1, .vcov=vcovHC, type="HC3")


## ----clust1, echo=T------------------------------------------------------
## load the multiwayvcov package
library(multiwayvcov)
## estimate a linear model of the log
## of strike volume on inflation, 
## unemployment and repression. 
mod1 <- lm(log(strike_vol + 1) ~ inflation +
    unemployment + sdlab_rep, data=strikes)
## Show the coefficient table using standard
## errors clustered on country code to 
## generate t-stats and p-values
coeftest(mod1, cluster.vcov, cluster=strikes$ccode)


## ----clust2, echo=T------------------------------------------------------
## You can also cluster on multiple variables, 
## in this case country and year. 
coeftest(mod1, cluster.vcov, cluster=strikes[,c("ccode", "year")])


## ----gsswt, echo=T-------------------------------------------------------
## read in the GSS 2012 data
gss <- import("GSS2012.dta")
## create a model without weighting
umod <- lm(realinc ~ educ*age + sex, data=gss)
## create a model using the weight argument to 
## specify weights (like the aweight argument in Stata)
wmod <- lm(realinc ~ educ*age + sex, data=gss, weight=gss$wtss)
## make a table of both models using stargazer
stargazer(umod, wmod, type="text")


## ----svyd, echo=T--------------------------------------------------------
## load the survey package
library(survey)
## Generate a vote for Obama variable
## valled voteob
gss$voteob <- car::recode(gss$pres08, "1=1; 2:4=0; else=NA")
## create a survey design argument that 
## treats the weightes in wtss as 
## probability weights
d <- svydesign(id=~1,strata=NULL, weights=~wtss, data=gss)
## estimate a survey weighted glm for Obama 
## vote (note: like using pweight in Stata)
smod <- svyglm(voteob ~ realinc + age + sex, design=d, family=binomial)
## summarize both models
summary(smod)
summary(wmod)


## ----wxt, echo=T---------------------------------------------------------
## Use weights as the count in the xtabs
## function to generate a weighted 
## cross-tab.  There are also functions
## in the survey package that do this. 
xtabs(wtss ~ pres08 + sex, data=gss)


## ----wtpkg1, echo=T------------------------------------------------------
## load the weights and dplyr packages
library(weights)
library(dplyr)
## generate datasets for male and female 
## respondents. 
xf <- filter(gss, sex == 2)
yf <- filter(gss, sex == 1)
## identify x, y and weight variables
x <- xf$realinc
wtx <- xf$wtss
y <- yf$realinc
wty <- yf$wtss
## Do a weighted t-test of income
## on sex from the GSS. 
wtd.t.test(x, y, wtx, wty)


## ----wtpkg2, echo=T------------------------------------------------------
## do a weighted correlation of 
## age and real income from the GSS
wtd.cor(log(gss$age), log(gss$realinc), gss$wtss)


## ----wtpkg3, echo=T------------------------------------------------------
## Calculate a weighted chi-squared
## statistic for the cross-tab of 
## presidential vote on sex. 
wtd.chi.sq(gss$pres08, gss$sex)


## ----devt, echo=TRUE, eval=F---------------------------------------------
## ## load the devtools package
## library(devtools)
## ## install the DAMisc package from my
## ## GitHub repository.
## install_github('davidaarmstrong/damisc')


## ----profile, echo=TRUE, eval=F------------------------------------------
## ## save the defaultPackages option
## dp <- getOption("defaultPackages")
## ## add foreign, MASS and DAMisc to the
## ## defaultPackages option
## options(defaultPackages=c(dp, "foreign", "MASS", "DAMisc"))


## ----chisq, echo=TRUE, results='hide'------------------------------------
## create a 2x2 matrix that has 100 and 25 
## in the first row and 3 and 11 in the
## second row. 
mat <- matrix(c(
   100, 25, 3, 11), ncol=2, byrow=T)
## Do the chi-squared test on the matrix
## we created above to show how warnings
## work. 
chisq.test(mat)


## ----cor1, echo=TRUE, eval=F---------------------------------------------
## ## generate a vector of integers between
## ## 1 and 5, note that you cold also have
## ## done this with x <- 1:5
## x <- c(1,2,3,4,5)
## ## generate a sequence of letters from a
## ## to e.
## y <- c("a", "b", "c", "d", "e")
## ## try to correlate letters and numbers
## ## to show how errors work.
## cor(x,y)


## ----simp, echo=TRUE-----------------------------------------------------
## show how to simulate p-values in 
## the chi-squared statistic. 
chisq.test(mat, simulate.p.value=TRUE)


## ----meancode, echo=TRUE-------------------------------------------------
## print the internals of the 
## mean function
mean


## ----meanmeth, echo=TRUE-------------------------------------------------
## show the classes of objects
## for which the mean is defined. 
methods("mean")


## ----meandef, echo=TRUE--------------------------------------------------
## print the internals of the 
## mean.default function, the 
## default method for calculating the
## mean
mean.default


## ----plotmeth, echo=TRUE-------------------------------------------------
## show all of the classes of objects
## for which a plot function is defined. 
methods("plot")


## ----pdf, echo=TRUE------------------------------------------------------
## show how to get code for any 
## function using getAnywhere
getAnywhere(plot.data.frame)


## ----plotincprest, echo=TRUE, results='hide', fig.show='hide'------------
## load the car package
library(car)
## create a scatter plot of y=prestige and
## x=income from the Duncan data using
## the formula interface
plot(prestige ~ income, data=Duncan)

## ----plotincprest2, echo=TRUE, fig.show='hide', fig.keep='none'----------
## make the same plot as above using 
## the x,y interface.  Note that the 
## only difference is how the axes are 
## labelled.
plot(Duncan$income, Duncan$prestige)


## ----univscatterex, echo=TRUE, fig.show='hide', fig.keep='high'----------
## The default plotting method for 
## a single numeric variable is a 
## univariate scatterplot
plot(Duncan$income)

## ----histex, echo=TRUE, fig.show='hide', fig.keep='high'-----------------
## The default plotting method for 
## a single categorical variable is a 
## bar plot
plot(Duncan$type)

## ----scatterex, echo=TRUE, fig.show='hide', fig.keep='high'--------------
## The default plotting method for 
## two numeric variables is a 
## bivariate scatterplot
plot(income ~ education, data=Duncan)

## ----twofacex, echo=TRUE, fig.show='hide', fig.keep='high'---------------
## The default plotting method for 
## two factors is a mosaic plot. 
## first, create a categorical income 
## variable so we have two categorical 
## variables to plot
Duncan$inc.cat <- cut(Duncan$income, 3)
## set the number of lines outside the plotting
## region to below=5, left=7, top=2, right=3
## to make room for plotting all of the labels. 
## las=2 plots labels perpendicular to their 
## respective axes. 
par(las=2, mar=c(5,7,2,3))
## make the mosaic plot
plot(inc.cat ~ type, data=Duncan, xlab="", ylab="")

## ----boxplotex, echo=TRUE, fig.show='hide', fig.keep='high'--------------
## The default plotting method for one 
## factor and one numeric variable
## is to make side-by-side boxplots. 
plot(income ~ type, data= Duncan, xlab="")


## ----p1, echo=TRUE, fig.show='hide', fig.keep='high'---------------------
## plot the scatterplot of income and prestige. 
## add x and y labels as well as a main label
## which appears on top of the plotting region. 
## pch=16 makes filled-in circles. 
plot(prestige ~ income, data=Duncan, pch=16, col="blue",
    xlab='% males earnings > $3500/year',
	ylab='% of NORC raters indicating profession as good or excellent',
	main = 'Prestige versus Income\n (Duncan Data)')


## ----p2, echo=TRUE, fig.show='hide', fig.keep='high'---------------------
## same plot as above, but changes the plotting 
## character to the numeric values underneath the 
## occupation type labels, which will be 1, 2 and 3, 
## for bc, prof and wc. 
plot(prestige ~ income, data=Duncan, pch=as.numeric(Duncan$type),
    xlab='% males earnings > $3500/year',
	ylab='% of NORC raters indicating profession as good or excellent',
	main = 'Prestige versus Income\n (Duncan Data)')


## ----p3, echo=TRUE, fig.show='hide', fig.keep='high'---------------------
## make a vector of three colors
cols <- c("blue", "black", "red")
## make same plot as above, but changing the color
## of the plotting symbols to 
## blue for blue collor, black for prof
## and red for white collar. 
plot(prestige ~ income, data=Duncan, pch=as.numeric(Duncan$type),
	col = cols[as.numeric(Duncan$type)],
    xlab='% males earnings > $3500/year',
	ylab='% of NORC raters indicating profession as good or excellent',
	main = 'Prestige versus Income\n (Duncan Data)')


## ----p4, echo=TRUE, fig.show='hide', fig.keep='high'---------------------
## create a function that makes a color
## ramp between light gray (gray75) and 
## dark gray (gray 25).  Note, gray100 is
## basically white and gray0 is basically 
## black. 
myRamp <- colorRampPalette(c("gray75", "gray25"))
## create a color vector from the ramp 
## that has as many values as there are 
## unique values in education. 
cols <- myRamp(length(unique(Duncan$education)))
## created a vector of plotting characters
## that is 15, 16 and 17 - filled in squares, 
## circles and triangles. 
pchs <- c(15,16,17)
## make a plot of prestige on income where
## the plotting characters are defined by 
## the occupation type and the colors are
## defined by education
plot(prestige ~ income, data=Duncan,
	pch=pchs[as.numeric(Duncan$type)],
	col = cols[order(Duncan$education)],
    xlab='% males earnings > $3500/year',
	ylab='% of NORC raters indicating profession as good or excellent',
	main = 'Prestige versus Income\n (Duncan Data)')


## ----addleg, echo=TRUE, fig.show='hide', fig.keep='high'-----------------
## create a vector of colors, blue, 
## black and red. 
cols <- c("blue", "black", "red")
## plot prestige on income defining plotting
## symbols and colors by occupation type. 
plot(prestige ~ income, data=Duncan, pch=as.numeric(Duncan$type),
	col = cols[as.numeric(Duncan$type)],
    xlab='% males earnings > $3500/year',
	ylab='% of NORC raters indicating profession as good or excellent',
	main = 'Prestige versus Income\n (Duncan Data)')
## add a legend to the bottom-right 
## corner of the plot that identifies whqt
## the plotting symbols and colors mean. 
legend("bottomright", c("Blue-collar", "Professional", "White-collar"),
    pch=1:3, col=cols, inset=.01)


## ----addregline, echo=TRUE, fig.show='hide', fig.keep='high'-------------
## make the same plot as immediately above. 
plot(prestige ~ income, data=Duncan, pch=as.numeric(Duncan$type),
  col = cols[as.numeric(Duncan$type)],
    xlab='% males earnings > $3500/year',
  ylab='% of NORC raters indicating profession as good or excellent',
  main = 'Prestige versus Income\n (Duncan Data)')
legend("bottomright", c("Blue-collar", "Professional", "White-collar"),
    pch=1:3, col=cols, inset=.01)


## ----addregline2, echo=TRUE, fig.show='hide', fig.keep='high'------------
## make the same plot as immediately above. 
plot(prestige ~ income, data=Duncan, pch=as.numeric(Duncan$type),
  col = cols[as.numeric(Duncan$type)],
    xlab='% males earnings > $3500/year',
  ylab='% of NORC raters indicating profession as good or excellent',
  main = 'Prestige versus Income\n (Duncan Data)')
legend("bottomright", c("Blue-collar", "Professional", "White-collar"),
    pch=1:3, col=cols, inset=.01)
## add a regression line for the blue-collar 
## occupations
abline(lm(prestige ~ income, data=Duncan,
	subset=Duncan$type == "bc"), col=cols[1])
## add a regression line for the professional
## occupations
abline(lm(prestige ~ income, data=Duncan,
	subset=Duncan$type == "prof"), col=cols[2])
## add a regression line for the white-
## collar occupations.
abline(lm(prestige ~ income, data=Duncan,
	subset=Duncan$type == "wc"), col=cols[3])


## ----idpts, echo=FALSE, fig.show='hide', fig.keep='high'-----------------
## make the same plot as above. 
plot(prestige ~ income, data=Duncan, pch=as.numeric(Duncan$type),
  col = cols[as.numeric(Duncan$type)],
    xlab='% males earnings > $3500/year',
  ylab='% of NORC raters indicating profession as good or excellent',
  main = 'Prestige versus Income\n (Duncan Data)')
legend("bottomright", c("Blue-collar", "Professional", "White-collar"),
    pch=1:3, col=cols, inset=.01)
abline(lm(prestige ~ income, data=Duncan,
  subset=Duncan$type == "bc"), col=cols[1])
abline(lm(prestige ~ income, data=Duncan,
  subset=Duncan$type == "prof"), col=cols[2])
abline(lm(prestige ~ income, data=Duncan,
  subset=Duncan$type == "wc"), col=cols[3])
## figure out which observations are the minister 
## and conductor.
inds <- which(rownames(Duncan) %in% c("minister", "conductor"))
## put the occupation labels to the left (pos=2) 
## of the points for the minister and conductor.
text(Duncan$income[inds], Duncan$prestige[inds], labels=rownames(Duncan)[inds], pos=2)


## ----hist1, echo=TRUE, fig.show='hide', fig.keep='high'------------------
## make a histogram of prestige with
## ten different bars. 
hist(Duncan$prestige, nclass=10, xlab="Prestige", main="")


## ----hist2, echo=TRUE, fig.show='hide', fig.keep='high'------------------
## make same graph as above, but fill 
## in the bars wkith light gray
hist(Duncan$prestige, nclass=10, xlab="Prestige",
	main="", freq=FALSE, col="gray80")
## add a density line to the graph.  Note that 
## for this to work, the argument freq=FALSE
## has to be used in the hist() function. 
lines(density(Duncan$prestige), col="red", lwd=1.5)


## ----bp1, echo=TRUE, fig.show='hide', fig.keep='high'--------------------
## load the dplyr package
library(dplyr)
## Use the Duncan data, group by the
## type variable, make a summary where 
## in the result, the variable mp, 
## is the mean of prestige within 
## occupation type. 
bp1 <- Duncan %>% group_by(type) %>% summarise(mp = mean(prestige))
## sort the result by average prestige. 
bp1 <- bp1[order(bp1$mp), ]
## make a bar plot of the sorted result. 
barplot(bp1$mp, horiz=F, names.arg = bp1$type)


## ----repregion, echo=TRUE------------------------------------------------
## load the gmodels package 
library(gmodels)
## Do the same kind of summary as above, but 
## this time include the lower and upper 
## onfidence bounds. 
ag <- Duncan %>% group_by(type) %>% summarize(mp = mean(prestige), 
    lwr = ci(prestige)[2], upr = ci(prestige)[3])
## print the summarized data
ag


## ----dp1, echo=TRUE, fig.show='hide', fig.keep='high'--------------------
## order the summarized data by 
## average prestige.  
ag <- ag[order(ag$mp), ]
## make a dot plot using the base 
## graphics function dotahcrt()
dotchart(ag$mp, ag$type, xlim=c(10,90),
	lcolor=NA, pch=16, xlab = "Prestige")
## add in the error bars using the base
## graphics function segments()
segments(ag$lwr, 1:3, ag$upr, 1:3)


## ----ggscat1a, echo=T----------------------------------------------------
## make a ggplot object using 
## income as x and prestige as y. 
g <- ggplot(Duncan, aes(x=income, y=prestige))


## ----ggscat1, echo=T, fig.height=8, fig.width=8, out.width=".5\\textwidth", fig.align="center"----
## add the points to the plot, use open circle
## plotting symbols. 
g <- g + geom_point(pch=1)
## print the plot
g


## ----ggscat2, echo=T, fig.height=8, fig.width=8, out.width=".5\\textwidth", fig.align="center"----
## add to the previous plot, a 
## black and white them and 
## change the aspect ratio to be 1. 
g <- g + theme_bw() + theme(aspect.ratio=1)
## print the plot
g


## ----ggscat3, echo=T, fig.height=8, fig.width=8, out.width=".5\\textwidth", fig.align="center"----
## initialize the previous plot with both
## colour and shape being a function of
## occupation type.  
## add in the points and the theme elements
## from above. 
g <-  ggplot(Duncan, aes(x=income, y=prestige, colour=type, shape=type)) + 
    geom_point() + theme_bw() + theme(aspect.ratio=1)
## print the plot.
g


## ----ggscat4, echo=T, fig.height=8, fig.width=8, out.width=".5\\textwidth", fig.align="center"----
## Make the same plot as above, but using 
## sale_shape_manual, make the plotting symbols
## open circles, squares and triangles. 
g <- ggplot(Duncan, aes(x=income, y=prestige, colour=type, shape=type)) + 
    geom_point() + theme_bw() + theme(aspect.ratio=1) + 
   scale_shape_manual(values=c(1,2,3))
## print the plot. 
g


## ----ggscat5, echo=T, fig.height=8, fig.width=8, out.width=".5\\textwidth", fig.align="center"----
## Same as above, but change the colors to 
## blue, black and red instead of the default
## colors with the scale_colour_manual() function. 
g <- ggplot(Duncan, aes(x=income, y=prestige, colour=type, shape=type)) + 
    geom_point() + theme_bw() + theme(aspect.ratio=1) + 
    scale_shape_manual(values=c(1,2,3), name="Occupation\nType", 
                       labels = c("Blue Collar", "Professional", "White Collar")) + 
    scale_colour_manual(values=c("blue", "black", "red"), name="Occupation\nType", 
                        labels = c("Blue Collar", "Professional", "White Collar"))
## print the plot
g


## ----ggscat6, echo=T, fig.height=8, fig.width=8, out.width=".5\\textwidth", fig.align="center"----
## add x- and y-axis to the plot. 
g <- ggplot(Duncan, aes(x=income, y=prestige, colour=type, shape=type)) + 
    geom_point() + theme_bw() + theme(aspect.ratio=1) + 
    scale_shape_manual(values=c(1,2,3)) + 
    scale_colour_manual(values=c("blue", "black", "red")) + 
    xlab('% males earnings > $3500/year') + 
    ylab('% of NORC raters indicating profession as good or excellent')
## print the plot. 
g


## ----ggscat7, echo=T, fig.height=8, fig.width=8, out.width=".5\\textwidth", fig.align="center"----
## add regression lines to the previous
## plot with the geom_smooth() function.  
## Note that the default method is a 
## LOESS smoother, so if you want a linear
## model, you need to use method="lm". 
## You can use se=FALSE, to turn off the 
## confidence bands. 
g <- ggplot(Duncan, aes(x=income, y=prestige, colour=type, shape=type)) + 
    geom_point() + geom_smooth(method="lm", se=FALSE, size=.5) +
    geom_smooth(method="loess", lty=2, se=FALSE, size=.25) +
    theme_bw() + theme(aspect.ratio=1) + 
    scale_shape_manual(values=c(1,2,3)) + 
    scale_colour_manual(values=c("blue", "black", "red")) + 
    xlab('% males earnings > $3500/year') + 
    ylab('% of NORC raters indicating profession as good or excellent')
## print the plot
g


## ----ggscat8, echo=T, fig.height=5, fig.width=10, out.width="\\textwidth", fig.align="center"----
## make the same plot as above, but juxtapose 
## the occupation type points and lines by
## using the facet_wrap() function. 
g <- ggplot(Duncan, aes(x=income, y=prestige)) + 
    geom_point() + geom_smooth(method="lm") +
    theme_bw() + theme(aspect.ratio=1) + 
    facet_wrap(~type) + 
    xlab('% males earnings > $3500/year') + 
    ylab('% of NORC raters indicating profession as good or excellent')
## print the plot
g


## ----ggbar1, echo=T, fig.height=8, fig.width=8, out.width=".5\\textwidth", fig.align="center"----
## create a bar plot using the 
## stat_summary function with the 
## bar geometry.  Note that fun.y
## defines what values are plotted
## as the heights of the bars. 
ggplot(Duncan, aes(x=type, y=prestige)) + 
    stat_summary(geom="bar", fun.y=median) + 
    theme_bw() 


## ----ggbar2, echo=T, fig.height=8, fig.width=8, out.width=".5\\textwidth", fig.align="center"----
## same as above, but use the reorder 
## function to re-order the type values
## based on the average prestige value
## within each occupation type. 
ggplot(Duncan, aes(x=reorder(type, prestige, mean), y=prestige)) + 
    stat_summary(geom="bar", fun.y=mean) + 
    theme_bw() 


## ----gg_bar2, echo=TRUE, fig.show='hide', fig.keep='high', fig.height=4, fig.width=10----
## make a bart plot of income based on region 
## from the Chile data.  The position = 
## position_dodge(1) moves the bars from 
## stacked to side-by-side. 
p <- ggplot(subset(Chile, !is.na(education)), 
      aes(x=reorder(region, income, mean, na.rm=TRUE), 
          y=income, fill=education)) + 
    stat_summary(fun.y=length,  geom="bar", position = position_dodge(1)) + 
    xlab("Region") + 
    theme_bw()
## print the plot. 
p


## ----gg_hist, echo=TRUE, fig.show='hide', fig.keep='high'----------------
## initialize a plot with prestige
## as x. 
p <- ggplot(Duncan, aes(prestige))
## add to the plot a histogram 
## geometry where y is the density
## within the group. 
p2 <- p + xlim(1,101) + geom_histogram(aes(y = ..density..),
	color="black", size=.25, fill="gray75", breaks=seq(1,101,by=10)) +
  theme_bw()  + theme(aspect.ratio=1)
## print the plot
p2


## ----gg_hist2, echo=TRUE, fig.show='hide', fig.keep='high'---------------
## add to the plot a density line. 
p2 + stat_density(geom = "line", col="red")


## ----gg_dot, echo=TRUE, fig.show='hide', fig.keep='high'-----------------
## initialize a plot with type on x
## and prestige on y.  
ggplot(Duncan, aes(x=reorder(type, prestige, mean), y=prestige)) + 
    # add an error-bar geometry
    # that uses the mean_cl_normal
    # function from the Hmisc package. 
    stat_summary(geom="errorbar", fun.data=mean_cl_normal, width=.1) + 
    # add points at the mean of 
    # prestige for each type. 
    stat_summary(geom="point", fun.y=mean) + 
    # add an x-label. 
    xlab("Occupation Type") 


## ----gg_dot2, echo=TRUE, fig.show='hide', fig.keep='high'----------------
## same plot as before, but use 
## scale_x_discrete to change the 
## tick-mark labels on the x-axis. 
ggplot(Duncan, aes(x=reorder(type, prestige, mean), y=prestige)) + 
    stat_summary(geom="errorbar", fun.data=mean_cl_normal, width=.1) + 
    stat_summary(geom="point", fun.y=mean) + 
    xlab("Occupation Type") + 
    scale_x_discrete(labels=c("Blue Collar", "White Collar", "Professional"))    


## ----fw1, echo=T, fig.height=8, fig.width=8, out.width=".5\\textwidth", fig.align="center"----
## load the car package
library(car)
## put the Ornstein data into 
## your workspace. 
data(Ornstein)
## make a scatterplot of the log of 
## assets on interlocks.  
## Use facet_wrap() to juxtapose
## the plots based on sector. 
ggplot(Ornstein, aes(x=log(assets), y=interlocks)) + 
    geom_point(pch=1) + facet_wrap(~sector)


## ----fg1, echo=T, fig.height=10, fig.width=6, out.width=".5\\textwidth", fig.align="center"----
## load the car package and 
## reload thhe Ornstein data. 
library(car)
data(Ornstein)
## make the same plot as above, 
## but make the facets a grid 
## based on sector and nation
ggplot(Ornstein, aes(x=log(assets), y=interlocks)) + 
    geom_point(pch=1) + 
    facet_grid(sector ~ nation)


## ----gg_juxt2, echo=TRUE, fig.show='hide', fig.keep='high', fig.height=8, fig.width=6----
## add regression lines to the 
## previous plot.
ggplot(Ornstein, aes(x=log(assets), y=interlocks)) + 
    geom_point(pch=1) + 
    facet_wrap(~sector) + 
    geom_smooth(method="lm", se=FALSE, fullrange=TRUE)


## ----estbothmods, echo=T-------------------------------------------------
## estimate a linear model of 
## prestige on income education 
## and type. 
mod1 <- lm(prestige ~ income + education + type, data=Duncan)
## estimate a linear model of 
## prestige on a third-degree 
## polynomial of income, education
## and type.  The goal is to compare
## the effects of income in the 
## two models. 
mod2 <- lm(prestige ~ poly(income, 3) + education + type, data=Duncan)


## ----effboth, echo=TRUE--------------------------------------------------
## calculate the effects of 
## both models. 
e1 <- predictorEffect("income", mod1, focal.levels=25)
e2 <- predictorEffect("income", mod2, focal.levels=25)


## ----edboth, echo=TRUE---------------------------------------------------
## make a data frame from the first effect
## with variables for x (income), 
## fitted values and lower and upper 
## confidence bounds. 
ed1 <- do.call("data.frame", e1[c("x", "fit", "lower", "upper" )])
## add a variable to the data frame 
## identify that these effects come 
## from the linear income variable. 
ed1$model <- factor(1, levels=1:2, labels=c("Linear", "Polynomial"))
## do the same as above for the second
## model effect, but identify these predictions
## as coming from the polynomial model. 
ed2 <- do.call("data.frame", e2[c("x", "fit", "lower", "upper" )])
ed2$model <- factor(2, levels=1:2, labels=c("Linear", "Polynomial"))


## ----appboth, echo=TRUE--------------------------------------------------
## use rbind to put the two 
## effect data frames together. 
plot.dat <- rbind(ed1, ed2)


## ----both1, echo=T, fig.height=8, fig.width=8, out.width=".5\\textwidth", fig.align="center"----
## make a plot of fit on income using 
## model as a grouping variable. 
## use the line geometry to make the effect
## line. 
g <- ggplot(plot.dat, aes(x=income, y=fit, colour=model)) + 
    geom_line() + 
    theme_bw() + 
    theme(aspect.ratio=1)
## print the plot
g


## ----both2, echo=T, fig.height=8, fig.width=8, out.width=".5\\textwidth", fig.align="center"----
## same as above, but include a geom_ribbon()
## function that will put the cofnidence 
## bounds around the effect line.  Note that
## the ribbon is plotted first, which will allow
## ther line to be plotted on top of the ribbon 
## making it easier to see. 
g <- ggplot(plot.dat, aes(x=income, y=fit, colour=model)) + 
    geom_ribbon(aes(ymin = lower, ymax=upper, fill=model), 
        alpha=.25, size=0) + 
    geom_line() + 
    ylab("Predicted Prestige") + 
    theme_bw() + 
    theme(aspect.ratio=1)
## print the plot
g


## ----countyread, echo=T--------------------------------------------------
## load the sf package
library(sf)
## read in the county boundaries geography
counties <- st_read("county_boundaries/cb_2018_us_county_20m.shp")
## look at the first six rows of the data. 
head(counties)


## ----crimeread, echo=T---------------------------------------------------
## load the rio package
library(rio)
## import the crime data. 
crime <- import("crime_data_2016.csv")


## ----changefips, echo=T--------------------------------------------------
## change the names of the state and 
## county fips code variables in the 
## crime data to match the names of those
## same variables in the county boundary 
## data. 
names(crime)[which(names(crime) == "FIPS_ST")] <- "STATEFP"
names(crime)[which(names(crime) == "FIPS_CTY")] <- "COUNTYFP"


## ----factoint, echo=T----------------------------------------------------
## change the state and county fips codes from 
## factors to integers by first converting them 
## into characters.  
counties$STATEFP <- as.integer(as.character(counties$STATEFP))
counties$COUNTYFP <- as.integer(as.character(counties$COUNTYFP))


## ----mergecty, echo=T----------------------------------------------------
## load the dplyr package
library(dplyr)
## merge the county boundaries 
## with the crime data only keeping those 
## observations that match up with the 
## county boundaries. 
counties <- left_join(counties, crime)


## ----noakhi, echo=T------------------------------------------------------
## find out which observations are not 
## in the continental US.  I had to find out
## the appropraite numbers from the codebook. 
## subset the data keeping only the continental 
## US regions. 
counties <- counties[-which(counties$STATEFP %in% c(2, 15, 66, 72)), ]


## ----tm1, echo=T, eval=F-------------------------------------------------
## ## load the tmap package.
library(tmap)
## generate the log of crime variable.
counties$logcrime <- log(counties$crime_rate_per_100000)
## create the map and fill in the regions based
## on the log of crime variable.
cmap <- tm_shape(counties) + 
  tm_fill(col="logcrime", 
    palette = "RdBu", 
    title="Log(Crime Rate/100000)")


tmap_leaflet(cmap)


## ----map1, echo=TRUE, fig.show='hide'------------------------------------
data(world, package="spData")
wbmig <- import("wbmig.dta")
library(countrycode)
world$ccode <- countrycode(world$iso_a2, "iso2c", "cown")
wbmig$ccode <- as.integer(wbmig$ccode)
wbmig <- subset(wbmig, !is.na(ccode))
world <- left_join(world, wbmig)
world$logmig <- log(world$wbpropmig)


## ----plotworld1, echo=T, eval=F------------------------------------------
## tm1 <- tm_shape(world) +
##     tm_fill(col="logmig", title="Log(Proportion of Migrants)") +
##     tm_borders(col="gray50", lwd=.5)


## ----eval=F--------------------------------------------------------------
## tmap_leaflet(tm1)


## ----convsp, echo=T------------------------------------------------------
worldsp <- as(world, "Spatial")
class(worldsp)


## ----adj, echo=T---------------------------------------------------------
library(spdep)
nl <- poly2nb(worldsp)
worldsp@data[172, ]
worldsp@data[nl[[172]], ]


## ----wmat, echo=T--------------------------------------------------------
wmat <- nb2listw(nl, zero.policy=TRUE)
print.listw(wmat, zero.policy=TRUE)


## ----moran, echo=T-------------------------------------------------------
moran.test(worldsp@data$logmig, wmat, zero.policy=TRUE, na.action=na.omit)


## ----moran2, echo=T------------------------------------------------------
moran.mc(worldsp@data$logmig, wmat, zero.policy=TRUE, na.action=na.omit, nsim=1000)


## ----locmoran, echo=T, eval=F--------------------------------------------
## worldsp2 <- as(world[which(!is.na(world$logmig)), ], "Spatial")
## l <- localmoran(worldsp2$logmig, nb2listw(poly2nb(worldsp2),
##     style="C", zero.policy=T), zero.policy=TRUE, na.action=na.omit)
## worldsp2@data$localI <- l[,1]

## ----locmorload, results='hide', echo=FALSE------------------------------
load("worldsf2.rda")


## ----locIplot, echo=T, eval=F--------------------------------------------
## worldsf2 <- as(worldsp2, "sf")
## tm_shape(worldsf2) + tm_fill(col="localI",
##     palette = "RdBu", title="Moran's I",
##     breaks= c(-Inf, -2, 0, 2, Inf)) + tm_borders() +
##     tm_layout(legend.position = c("left", "center"))


## ----worldreg, echo=T----------------------------------------------------
data(world)
world <- as(world, "Spatial")
## find neighbors
nl <- poly2nb(world)
## make list of weights
wmat <- nb2listw(nl, zero.policy=TRUE)
localStat <- function(x, neigh, stat="mean", ...){
    out <- sapply(nl, function(z)do.call(stat, list(x=x[z], ...)))
    return(out)
}
world@data$lifeExp_L1 <- localStat(world@data$lifeExp, nl, na.rm=T)
slmod <- lm(lifeExp ~ lifeExp_L1 + log(gdpPercap) + log(pop), data=world@data)
summary(slmod)


## ----worldreg2, echo=T---------------------------------------------------
summary(slmod2 <- lagsarlm(lifeExp ~ log(pop) + log(gdpPercap), data=world@data, 
    listw = wmat, zero.policy=T))


## ----star1, echo=T-------------------------------------------------------
library(rio)
strikes <- import("https://quantoid.net/files/rbe/strikes_small.rda")
library(stargazer)
mod1 <- lm(log(strike_vol + 1) ~ inflation +
    unemployment + sdlab_rep, data=strikes)
mod2 <- lm(log(strike_vol + 1) ~ union_cent +
    unemployment + sdlab_rep, data=strikes)
stargazer(mod1, mod2, type="text")


## ----star2, echo=T-------------------------------------------------------
stargazer(mod1, mod2, type="text", notes.append=F,
  star.cutoffs = .05, notes="*p < 0.05, two-sided",
  covariate.labels=c("Inflation", "Union Percentage",
    "Unemployment Rate", "Soc.Dem./Labor Percentage in Legis.",
    "Intercept"))


## ----expstar, echo=T, results='hide'-------------------------------------
stargazer(mod1, mod2, type="text", notes.append=F,
    out = "table.html", star.cutoffs = .05, 
    notes="*p < 0.05, two-sided",
    covariate.labels=c("Inflation", "Union Percentage",
    "Unemployment Rate", "Soc.Dem./Labor Percentage in Legis.",
    "Intercept"), dep.var.labels = "log(Strikes)")


## ----robse, echo=T-------------------------------------------------------
library(stargazer)
library(sandwich)
library(lmtest)
mod1 <- lm(log(strike_vol + 1) ~ inflation +
    unemployment + sdlab_rep, data=strikes)
mod1 <- lm(log(strike_vol + 1) ~ inflation +
    unemployment + sdlab_rep, data=strikes)
mod2 <- lm(log(strike_vol + 1) ~ union_cent +
    unemployment + sdlab_rep, data=strikes)
se1 <- mod1 %>% vcovHC(., type="HC3") %>% diag %>% sqrt
## or
## se1 <- sqrt(diag(vcovHC(mod1, type="HC3")))
se2 <- mod2 %>% vcovHC(., type="HC3") %>% diag %>% sqrt


## ----robstar, echo=T-----------------------------------------------------
library(stargazer)
stargazer(mod1, mod2, type="text", notes.append=F,
    star.cutoffs = .05, 
    notes="*p < 0.05, two-sided",
    covariate.labels=c("Inflation", "Union Percentage",
    "Unemployment Rate", "Soc.Dem./Labor Percentage in Legis.",
    "Intercept"), se=list(se1, se2))


## ----rv1, echo=T---------------------------------------------------------
library(rvest)
## read in the data
india <- read_html("https://en.wikipedia.org/wiki/List_of_political_parties_in_India") 
## pull out all of the tables. 
tabs <- html_table(india, fill=T)
## look at the third element of 
head(tabs[[3]])


## ----rv2, echo=T---------------------------------------------------------
library(rvest)
## read in the data
uk <- read_html("https://en.wikipedia.org/wiki/List_of_Prime_Ministers_of_the_United_Kingdom") 
## pull out all of the tables. 
tabs <- html_table(uk, fill=T)
## set the column names of the object
colnames(tabs[[2]]) <- c("", "Portrait", "PM", "Start", "End", 
    "Year", "portfolio", "party", "ministry", "monarch", "note")
## take out the first row which was the column names before
tabs[[2]] <- tabs[[2]][-1, ]
## look at the first 2 rows
tabs[[2]][1:2, ]


## ----readafg, echo=T-----------------------------------------------------
## load the rvest package
library(rvest)
## read in the website data. Note
## that this will not allow you to see
## the website directly, even if you 
## type h and hit enter. 
h <- read_html("https://www.state.gov/reports/2018-country-reports-on-human-rights-practices/afghanistan/")


## ----cb, echo=T----------------------------------------------------------
## extract the text from the html file.  
## the html_node() function identifies which 
## part of the document to focus in on. 
## Once the focus is set, that result is piped
## to the html_text() function which extracts
## all of the text within that block.
text <- html_node(h, xpath = "//div[starts-with(@class, 'report__content')]") %>% html_text


## ----makedf, echo=T------------------------------------------------------
## load quanteda package
library(quanteda)
## use dfm to make a document-feature
## matrix.  The arguments allow you to 
## turn on and off a bunch of pre-processing
## steps. 
dfmat <- dfm(text, 
    tolower=TRUE, remove=stopwords(), 
    remove_numbers=TRUE, remove_punct=TRUE, 
    remove_symbols=TRUE, ngrams=1)
## identifies the top 10 most common
## words used in the document(s). 
topfeatures(dfmat, n=10)


## ----bardf, echo=T, fig.show='hide', fig.height=8, fig.width=8-----------
## load the tibble package, it provides 
## a different way to make data frames. 
library(tibble)
## sort the top features in decreasing 
## fashion and store the sorted result
## in x. 
x <- sort(topfeatures(dfmat, n=10), decreasing=T)
## make a data frame (using tibble) where
## label is the name of the word 
## from the topfeatures() function and 
## y is the count for that word
tmp <- tibble(
    label = factor(names(x), levels=names(x)),  
    y=x)
## initialize a ggplot object using the
## data you just created where label is x 
## and y (the count) is y. 
g <- ggplot(data=tmp, aes(x=label, y=y))
## create a bar plot of the counts
g + geom_bar(stat="identity") + xlab("") + 
    theme_bw() + ylab("# Mentions") + 
    # here, we can turn the axis tick-mark 
    # labels to 45 degrees to prevent 
    # overplotting. 
    theme(axis.text.x = 
        element_text(angle = 45, hjust = 1)) 


## ----for1, echo=T--------------------------------------------------------
## create a vector of words
n <- c("one", "two", "three", "four")
## open a for loop, the syntax here is 
## for(<counter> in <vector of values>)
## the vector of values doesn't have to 
## be numbers, but it often is. 
for(i in 1:4){
    # print the ith value of the 
    # n-vector of words. 
    print(n[i])
## close the loop with a brace. 
}


## ----for1_hand, echo=T---------------------------------------------------
## show how to get the same result 
## as the previous loop manually. 
n[1]
n[2]
n[3]
n[4]


## ----for2, echo=T--------------------------------------------------------
## open a for loop - showing that the 
## counter doesn't have to be a single
## letter like "i", here it's "fred"
for(fred in 1:4){
    print(n[fred])
}


## ----while, echo=T-------------------------------------------------------
## initialize a counter "i" at zero
i <- 0
## initialize an object "sumpois"
## at zero. 
sumpois <- 0
## the while function is also a loop, but
## it continues to execute until a 
## certain condition is met.  Here, 
## we're summing poisson random variates
## until they are greater than or equal to 
## 100 (because the while loop continues to 
## execute while sumpois is less than 100). 
while(sumpois < 100){
    # increment the counter
    i <- i+1
    # add a random poisson(3) draw to the 
    # previous sumpois objet
    sumpois <- sumpois + rpois(1, 3)
## close the loop
}
## print the value of i
i


## ----permtest, echo=T----------------------------------------------------
## load the vcd package
library(vcd)
## define x as sector from the ornstein data
x <- Ornstein$sector
## define y as nation from the ornstein data
y <- Ornstein$nation
## initialize a vector of missing values
## that is 2000 long. 
res <- rep(NA, 2000)
## loop over i from 1 to 2000
for(i in 1:2000){
    # make a temporary x value that is 
    # our original x-values randomly 
    # rearranged. 
    tmpX <- sample(x, length(x),replace=F)
    # find the cramer's v from the 
    # original y and the reordered x; 
    # store the result in the ith value of res
    res[i] <- assocstats(table(tmpX, y))$cramer
## close the loop
}
## now res will have the null distribution of
## cramer's v statistics. We want to find out 
## what proportion of these null-relationship
## random draws is bigger than the cramer's v
## stat from the original data.  This is the 
## p-value. 
mean(res > assocstats(table(x,y))$cramer)


## ----permgraph, echo=T, fig.show='hide'----------------------------------
## plot the density of res, make sure that 
## the x-axis extends long enough to capture
## all of the values in res and the observed
## cramer's v value. 
plot(density(res), xlim=range(c(res, assocstats(table(x,y))$cramer)))
## put a vertical line at the cramer's v
## value from the original data. 
abline(v=assocstats(table(x,y))$cramer, lty=3)


## ------------------------------------------------------------------------
## read in the main page that has all of the country links
## on it. 
h1 <- read_html("https://www.state.gov/reports/2018-country-reports-on-human-rights-practices")
## use the html_nodes() function to to find all of the 
## option tags. Use the html_attr() function to extract the 
## value element which is the link. 
links <- html_nodes(h1, xpath = "//option[starts-with(@value, 'https://www.state.gov/reports/2018-country-reports-on-human-rights-practices')]") %>% html_attr("value")
## do the same as above to identify the options, but here
## use the html_text() function to grab the country name, i.e., 
## the text that is inside the option tag. 
countries <- html_nodes(h1, xpath = "//option[starts-with(@value, 'https://www.state.gov/reports/2018-country-reports-on-human-rights-practices')]") %>% html_text


## ----get_texts, echo=T, eval=F-------------------------------------------
## ## initialize a list
## texts <- list()
## 
## ## loop over the number of links
## for(i in 1:length(links)){
## 
##     ## read in the url in the ith link
##     h1 <- read_html(links[i])
## 
##     ## pull the text from the <div> that has a class that starts with 'report_content'
##     texts[[i]] <- html_node(h1, xpath = "//div[starts-with(@class, 'report__content')]") %>% html_text
## }
## ## pue the country names with the texts.
## names(texts) <- countries


## ----sd_all_dfm, echo=T, eval=F------------------------------------------
## ## make the texts into a vector
## texts <- do.call("c", texts)
## ## load the quanteda package
## library(quanteda)
## ## make a document-feature matrix from all
## ## of the texts,
## dfmat <- dfm(texts,
##     tolower=TRUE, remove=stopwords(),
##     remove_numbers=TRUE, remove_punct=TRUE,
##     remove_symbols=TRUE, ngrams=1)


## ----ifelse, echo=TRUE---------------------------------------------------
## make a vector of the integers 1-5. 
x <- c(1, 2, 3,4,5)
## if x is less than three, return zero, 
## otherwise return one. 
ifelse(x < 3, 0, 1)


## ----ifelse2, echo=TRUE--------------------------------------------------
## make a vector of the integers 1-5. 
x <- c(1, 2, 3,4,5)
## if x is less than three, return zero, 
## otherwise return the value of x 
ifelse(x < 3, 0, x)


## ----ifthen, echo=TRUE---------------------------------------------------
## load the car package
library(car)
## make an object tmp which is a
## data frame with variables income 
## and type. 
tmp <- Duncan[,c("income", "type")]
## change the third and fortieth values of 
## income to missing (NA). 
tmp$income[c(3,40)] <- NA
## make a variable income_imp that is a 
## copy of the income variable. 
tmp$income_imp <- tmp$income
## loop over the number of rows in tmp 
for(i in 1:nrow(tmp)){
    # evaluate whether the ith value of 
    # tmp$income is missing
    if(is.na(tmp$income[i])){
        # if it is missing, fill in 
        # the ith value of tmp$income_imp
        # with a randomly drawn value using
        # sample. 
        tmp$income_imp[i] <- sample(
            # the values you're sampling are 
            # the valid values of income from 
            # occupations with the same type. 
            na.omit(tmp$income[which(tmp$type == tmp$type[i])]), 1)
    # close the if loop
    }
    # open the else loop
    else{
    # here, the else loop is blank because we 
    # don't want to do anything if the ith value 
    # of tmp$income isn't missing.  We could just
    # as easily omitted the else block altogether. 
    # close the else block
    }
## close the for loop
}


## ----apply, echo=T-------------------------------------------------------
## set the random number generator seed. 
set.seed(10)
## make a 5x5 matrix filled in with 
## randomly drawn values between 1 and 
## 5 (not necessarily integers). 
mat <- matrix(runif(25,1,5), ncol=5)
## create the means within each column
## by using apply
colmeans <- apply(mat, 2, mean)
## print the colmeans object
colmeans
## create the means within each row. 
rowmeans <- apply(mat, 1, mean)
## print the rowmeans object
rowmeans
## note that there are functions colMeans and 
## rowMeans that already do this in abit
## simpler fashion. 


## ----apply_equiv, echo=T-------------------------------------------------
## show how the same thing as above could be 
## accomplished with a mnore general function
colmeans <- apply(mat, 2, function(z)mean(z))
colmeans
rowmeans <- apply(mat, 1, function(z)mean(z))
rowmeans


## ----applyq, echo=T------------------------------------------------------
## load the car package
library(car)
## get the 25th, 50th and 75th percentiles
## of the income, education and prestige
## variables from the Duncan data, 
apply(Duncan[,c("income", "education", "prestige")],
    2, quantile, c(.25,.5, .75), na.rm=T)


## ----by1, echo=T---------------------------------------------------------
## use by to calculate the mean of education
## by occuption type from the Duncan data
by1 <- by(Duncan$education, list(Duncan$type), mean)
## print the result. 
by1


## ----b1vec, echo=T-------------------------------------------------------
## turn by1 into a vector instead of a list
by1.vec <- c(by1)
## print the vector. 
by1.vec


## ----by2, echo=T---------------------------------------------------------
## Calculate the means of prestige, income and education
## by occupation type using a combination of by and 
## apply
by2 <- by(Duncan[,c("prestige", "income", "education")],
    list(Duncan$type),
    function(x)apply(x, 2, mean))
## print the result. 
by2


## ----by2mat, echo=T------------------------------------------------------
## turn the result from by2 into a matrix instead of a list
mat <- do.call(rbind, by2)
## print the matrix
mat


## ----bylm, echo=T--------------------------------------------------------
## use by to estimate models of prestige on 
## icnome and education for each occupation type
## separately.  Note that the dataset is the first
## argument and it essentially gets subset for 
## each different value of type. 
mods <- by(Duncan, list(Duncan$type),
    function(x)lm(prestige ~ income + education, data=x))

## print the models
mods
## summarize the first model.
summary(mods[[1]])


## ----applies1, echo=T----------------------------------------------------
## use lapply to get the model coefficients
## lapply always returns a list. 
lapply(mods, coef)
## use sapply to get the model coefficients
## sapply will return the simplest structure
## possible for the resulting data. In this 
## case it would be a matrix
sapply(mods, coef)


## ----meanfun, echo=T-----------------------------------------------------
mymean <- function(x){
	# calculate the sum
	sum.x <- sum(x, na.rm=T)
	# find the N by counting up all of the non-missing entries
	n.x <- sum(!is.na(x))
	# calculate the mean as the sum divided by n
	mean.x <- sum.x/n.x
	# print the mean
	mean.x
}


## ----calc_mymean, echo=T-------------------------------------------------
## make a vector of the integers 1 to 6
z <- 1:6
## use the function we created above to 
## calculate the mean
mymean(z)
## use R's built-in mean function to 
## calculate the mean
mean(z)


## ----ct2, echo=T---------------------------------------------------------
## define a function called CrossTable2
## it has arguments x and y, two variables. 
CrossTable2 <- function(x,y){
## make sure that the gmodels package is loaded. 
require("gmodels")
## create a CrossTable of x and y with all of the 
## appropriate values turned on and off. 
    CrossTable(x=x,y=y, prop.r=F, prop.t=F, prop.chisq=F,chisq=T,
        format="SPSS")
## close function block
}


## ----ct_duncan, echo=T---------------------------------------------------
## load the car package
library(car)
## bring the Duncan data into the workspace
data(Duncan)
## create a categorical education variable. 
Duncan$ed.cat <- cut(Duncan$education, 2)
## Try out the CrossTable2 function on 
## education categories and type
CrossTable2(Duncan$ed.cat, Duncan$type)


## ----firstfun, echo=T, eval=F--------------------------------------------
## ## make a .First function that takes no
## ## arguments.  This functino should
## ## be in a file called .RProfile
## .First <- function(){
##     # load a package; you could also do
##     # this with the defaultPackages option.
## 	library(foreign)
## 	# set the working directory automatically
##     Bdir <- "C:/Users/armstrod/Dropbox/IntroR/Boulder/"
## }

